# https://gist.github.com/jonathanmoore/2640302
from AccessControl.SecurityManagement import newSecurityManager
from Acquisition import aq_parent
from BTrees.OOBTree import OOBTree
from castle.cms.social import COUNT_ANNOTATION_KEY
from castle.cms.utils import clear_object_cache
from castle.cms.utils import index_in_es
from castle.cms.utils import retriable
from multiprocessing import Pool
from plone.app.redirector.interfaces import IRedirectionStorage
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import defaultpage
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from tendo import singleton
from urllib import quote_plus
from zope.annotation.interfaces import IAnnotations
from zope.component import getUtility
from zope.component.hooks import setSite

import itertools
import json
import logging
import requests
import time
import transaction


logger = logging.getLogger('castle.cms')


def get_facebook_url_data(urls):
    urls_string = '("%s")' % (
        '","'.join(quote_plus(url) for url in urls)
    )
    access_url = COUNT_URLS['facebook']['url'] % urls_string
    resp = requests.get(access_url).content
    data = json.loads(resp)
    count = 0
    if 'data' not in data:
        return 0
    data = data['data']
    for group in data:
        count += group.get('like_count', 0)
        count += group.get('total_count', 0)
        count += group.get('share_count', 0)
        count += group.get('comment_count', 0)
    return count


COUNT_URLS = {
    'pinterist': {
        'url': 'http://api.pinterest.com/v1/urls/count.json?callback=foobar&url=%s',
        'slash_matters': True
    },
    'linkedin': {
        'url': 'https://www.linkedin.com/countserv/count/share?format=json&url=%s'
    },
    'facebook': {
        'url': 'https://graph.facebook.com/fql?q=SELECT like_count, total_count, '
               'share_count,click_count,comment_count FROM link_stat '
               'WHERE url in %s',
        'generator': get_facebook_url_data
    }
}


def _get_url_data(args):
    config, url = args
    access_url = config['url'] % url
    resp = requests.get(access_url).content.lstrip('foobar(').rstrip(')')
    data = json.loads(resp)
    try:
        return data['count']
    except KeyError:
        return 0


_req_pool = Pool(6)
def _get_urls_data(args):
    results = []
    config, urls = args
    if config.get('generator'):
        results.append(config['generator'](urls))
    else:
        req_urls = []
        for orig_url in urls:
            req_urls.append((config, orig_url))
            if config.get('slash_matters'):
                req_urls.append((config, orig_url + '/'))
                req_urls.append((config, _swap_protocol(orig_url + '/')))
        results = _req_pool.map(_get_url_data, req_urls)
    return results


def _swap_protocol(url):
    if 'http://' in url:
        return url.replace('http://', 'https://')
    else:
        return url.replace('https://', 'http://')


def _get_urls(urls):
    additional = []
    for url in urls:
        additional.append(_swap_protocol(url))
    urls.extend(additional)
    return urls


# _pool = Pool(processes=3)
def _get_counts(urls):
    counts = {}
    type_order = []
    args_list = []
    for type_, config in COUNT_URLS.items():
        counts[type_] = 0
        type_order.append(type_)
        args_list.append((config, urls))

    for idx, type_count in enumerate(map(_get_urls_data, args_list)):
        type_ = type_order[idx]
        for type_count in type_count:
            counts[type_] += type_count
    return counts


def _merge_counts(one, two):
    for key, count in two.items():
        if key in one:
            one[key] += count
        else:
            one[key] = count
    return one


def _has_data(data):
    found = False
    for key, val in data.items():
        if val > 0:
            found = True
            break
    return found


def _count_diff(existing, new):
    diff = False
    for name, value in new.items():
        if value != existing.get(name):
            diff = True
            break
    return diff


@retriable(sync=True)
def get_social_counts(site, obj, site_url, count=0):
    counts = {}
    site_path = '/'.join(site.getPhysicalPath())
    obj_path = '/'.join(obj.getPhysicalPath())
    rel_path = obj_path[len(site_path):].strip('/')
    print('Looking up ' + rel_path)

    urls = [site_url.rstrip('/') + '/' + rel_path]
    registry = getUtility(IRegistry)
    if obj.portal_type in registry.get('plone.types_use_view_action_in_listings', []):
        urls.append(urls[0] + '/view')

    container = aq_parent(obj)
    if defaultpage.is_default_page(container, obj):
        container_path = '/'.join(container.getPhysicalPath())
        rel_path = container_path[len(site_path):].strip('/')
        urls.append(site_url.rstrip('/') + '/' + rel_path)

    redirector = getUtility(IRedirectionStorage)
    for redirect in redirector.redirects(obj_path):
        rel_path = redirect[len(site_path):].strip('/')
        urls.append(site_url.rstrip('/') + '/' + rel_path)

    urls = _get_urls(urls)
    counts = _get_counts(urls)

    if not _has_data(counts):
        return

    obj._p_jar.sync()
    annotations = IAnnotations(obj)
    existing = annotations.get(COUNT_ANNOTATION_KEY, OOBTree())

    if not _count_diff(existing, counts):
        return

    # XXX check if value different first before transaction!
    existing.update(counts)
    annotations[COUNT_ANNOTATION_KEY] = existing

    transaction.commit()

    index_in_es(obj)
    if count % 200 == 0:
        clear_object_cache(site)


def retrieve(site):
    setSite(site)
    registry = getUtility(IRegistry)
    site_url = registry.get('plone.public_url')
    if not site_url:
        return
    catalog = getToolByName(site, 'portal_catalog')
    count = 0
    for brain in catalog(review_state='published'):
        # we ignore some types...
        if brain.portal_type in ('Image', 'Dashboard',):
            continue
        path = brain.getPath()
        count += 1
        try:
            obj = brain.getObject()
            get_social_counts(site, obj, site_url, count)
            logger.info('retrieved social stats for: %s' % path)
        except:
            logger.warn('error getting social count totals for: %s' % path,
                        exc_info=True)
        time.sleep(2)


def run(app):
    singleton.SingleInstance('socialcounts')

    user = app.acl_users.getUser('admin')  # noqa
    newSecurityManager(None, user.__of__(app.acl_users))  # noqa

    for oid in app.objectIds():  # noqa
        obj = app[oid]  # noqa
        if IPloneSiteRoot.providedBy(obj):
            retrieve(obj)


if __name__ == '__main__':
    run(app)  # noqa



from plone.memoize.view import memoize
from plone.app.theming.utils import theming_policy
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFPlone.browser.main_template import MainTemplate as BaseMainTemplate
from castle.cms.theming import isPloneTheme


class MainTemplate(BaseMainTemplate):
    _ajax_template = ViewPageTemplateFile('templates/ajax_main_template.pt')
    _main_template = ViewPageTemplateFile('templates/main_template.pt')

    @property
    @memoize
    def is_castle_theme(self):
        policy = theming_policy(self.request)
        settings = policy.getSettings()
        if settings is None:
            return False
        if not policy.isThemeEnabled():
            return False
        if not isPloneTheme(settings):
            return True
        return False

    @property
    def ajax_template(self):
        if self.is_castle_theme:
            return self._ajax_template
        return BaseMainTemplate.ajax_template

    @property
    def main_template(self):
        if self.is_castle_theme:
            return self._main_template
        return BaseMainTemplate.main_template



# -*- coding: utf-8 -*-
from plone.uuid.interfaces import IUUID
from plone.app.contenttypes.interfaces import IFile
from . import cloudflare
from App.config import getConfiguration
from castle.cms.linkintegrity import get_content_links
from plone import api
from plone.app.imaging.utils import getAllowedSizes
from plone.cachepurging.hooks import KEY
from plone.cachepurging.interfaces import ICachePurgingSettings
from plone.cachepurging.interfaces import IPurger
from plone.cachepurging.utils import getPathsToPurge
from plone.cachepurging.utils import getURLsToPurge
from plone.namedfile.interfaces import IImageScaleTraversable
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.defaultpage import get_default_page
from Products.Five import BrowserView
from z3c.caching.interfaces import IPurgeEvent
from z3c.caching.interfaces import IPurgePaths
from zope.annotation.interfaces import IAnnotations
from zope.component import adapter
from zope.component import adapts
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryUtility
from zope.globalrequest import getRequest
from zope.interface import implements
from zope.testing.cleanup import addCleanUp
from ZPublisher.interfaces import IPubSuccess
from collective.documentviewer.settings import Settings as DocViewerSettings

import atexit
import logging
import Queue
import threading


logger = logging.getLogger('castle.cms')


@adapter(IPurgeEvent)
def queuePurge(event, force=False):
    """ so this is a little wonky here...
    We need to also purge here because plone.cachepurging will only update
    paths if caching proxies are defined. The deal here is that with
    cloudflare, we do not want to define caching proxies or we may not be """

    request = getRequest()
    if request is None:
        return

    annotations = IAnnotations(request, None)
    if annotations is None:
        return

    registry = queryUtility(IRegistry)
    if registry is None:
        return

    settings = registry.forInterface(ICachePurgingSettings, check=False)
    if not settings.enabled:
        return

    # THIS IS THE OVERRIDE HERE!!!
    # the original event goes forward IF there are cache proxies defined.
    # so we check if they are NOT and then only force puring
    if bool(settings.cachingProxies) and not force:
        return

    paths = annotations.setdefault(KEY, set())
    paths.update(getPathsToPurge(event.object, request))


@adapter(IPubSuccess)
def purge(event):
    """
    Asynchronously send PURGE requests.
    this is mostly copied from plone.cachingpurging
    """
    request = event.request

    annotations = IAnnotations(request, None)
    if annotations is None:
        return

    paths = annotations.get(KEY, None)
    if paths is None:
        return

    registry = queryUtility(IRegistry)
    if registry is None:
        return

    settings = registry.forInterface(ICachePurgingSettings, check=False)
    if not settings.enabled:
        return

    if paths:
        urls = []
        cf = cloudflare.get()
        if cf.enabled:
            for path in paths:
                urls.extend(cf.getUrlsToPurge(path))

            CastlePurger.purgeAsync(urls, cf)


class CastlePurgerFactory(object):

    implements(IPurger)

    def __init__(self, backlog=200):
        self.worker = None
        self.queue = None
        self.backlog = backlog
        self.queueLock = threading.Lock()

    def purgeAsync(self, urls, purger):
        if self.worker is None:
            self.queue = Queue.Queue(self.backlog)
            self.worker = Worker(self.queue, self)
            self.worker.start()
        try:
            self.queue.put((urls, purger), block=False)
            logger.debug('Queued %s' % ','.join(urls))
        except Queue.Full:
            # Make a loud noise. Ideally the queue size would be
            # user-configurable - but the more likely case is that the purge
            # host is down.
            if not getConfiguration().debug_mode:
                logger.warning("The purge queue for the URL %s is full - the "
                               "request will be discarded.  Please check the "
                               "server is reachable, or disable this purge "
                               "host", ','.join(urls))

    def purgeSync(self, urls, purger):
        resp = purger.purge(urls)
        logger.debug('Finished purge for %s' % ','.join(urls))
        return resp

    def stopThreads(self, wait=False):
        if self.worker is None:
            return
        self.worker.stopping = True
        try:
            self.queue.put(None, block=False)
        except Queue.Full:
            # no problem - self.stopping should be seen.
            pass
        ok = True
        if wait:
            self.worker.join(5)
            if self.worker.isAlive():
                logger.warning("Cloudflare worker thread failed to terminate")
                ok = False
        return ok


class Worker(threading.Thread):
    """Worker thread for purging.
    """

    def __init__(self, queue, producer):
        self.producer = producer
        self.queue = queue
        self.stopping = False
        super(Worker, self).__init__(name="PurgeThread for castle")

    def stop(self):
        self.stopping = True

    def run(self):
        logger.debug("%s starting", self)
        # Queue should always exist!
        q = self.producer.queue
        atexit.register(self.stop)
        try:
            while not self.stopping:
                item = q.get()
                if self.stopping or item is None:  # Shut down thread signal
                    logger.debug('Stopping worker thread for '
                                 '(%s, %s).' % (self.host, self.scheme))
                    break
                urls, purger = item

                try:
                    self.producer.purgeSync(urls, purger)
                    # XXX check valid response
                except Exception:
                    logger.exception('Failed to purge %s', ','.join(urls))
        except:  # FIXME: blind except
            logger.exception('Exception in worker thread '
                             'for (%s, %s)' % (self.host, self.scheme))
        logger.debug("%s terminating", self)

CastlePurger = CastlePurgerFactory()
def stopThreads():
    purger = CastlePurger
    purger.stopThreads()
addCleanUp(stopThreads)
del addCleanUp


class Purge(BrowserView):
    cf_enabled = False
    proxy_enabled = False

    def purge(self):
        site_path = '/'.join(api.portal.get().getPhysicalPath())
        cf = cloudflare.get()
        paths = []
        urls = []

        context = self.context
        dp = get_default_page(context)
        if dp:
            try:
                context = context[dp]
            except:
                pass
        objs = [context]
        for ref in get_content_links(context):
            try:
                objs.append(ref.to_object)
            except:
                pass

        for obj in objs:
            paths.extend(getPathsToPurge(obj, self.request))

        for path in paths:
            urls.extend(cf.getUrlsToPurge(path))

        urls = list(set(urls))

        registry = getUtility(IRegistry)
        settings = registry.forInterface(ICachePurgingSettings, check=False)
        purger = queryUtility(IPurger)
        if purger and settings.cachingProxies:
            self.proxy_enabled = True
            for path in paths:
                for url in getURLsToPurge(path, settings.cachingProxies):
                    purger.purgeAsync(url)

        success = True
        if cf.enabled:
            self.cf_enabled = True
            resp = CastlePurger.purgeSync(urls, cf)
            success = resp.json()['success']

        nice_paths = []
        for path in paths:
            if 'VirtualHostRoot' in path:
                path = path.split('VirtualHostRoot')[-1]
            else:
                path = path[len(site_path):]
            nice_paths.append(path)

        return nice_paths, success

    def __call__(self):
        self.success = False
        self.purged = []
        authenticator = getMultiAdapter((self.context, self.request),
                                        name=u"authenticator")
        if authenticator.verify():
            self.purged, self.success = self.purge()
        return self.index()


class LeadImagePurgePaths(object):
    """Paths to purge for lead images

    Includes:

    * ${object_path}/@@images/image
    * ${object_path}/@@images/image/mini etc

    """

    implements(IPurgePaths)
    adapts(IImageScaleTraversable)

    def __init__(self, context):
        self.context = context

    def getRelativePaths(self):
        if not getattr(self.context, 'image', None):
            return []

        prefix = '/' + self.context.virtual_url_path()
        paths = [prefix + '/@@images/image']

        for scale_name in getAllowedSizes().keys():
            paths.append(prefix + '/@@images/image/' + scale_name)

        return paths

    def getAbsolutePaths(self):
        return []


class DocViewerPurgePaths(object):
    """
    Paths to purge for person objects

    - purges person specific paths
    - and all objects inside folder

    """

    implements(IPurgePaths)
    adapts(IFile)

    def __init__(self, context):
        self.context = context

    def getRelativePaths(self):
        if self.context.getLayout() != 'documentviewer':
            return

        site_prefix = api.portal.get().virtual_url_path()
        uid = IUUID(self.context)
        base_image_path = site_prefix + '/@@dvpdffiles/{}/{}/{}'.format(
            uid[0],
            uid[1],
            uid
        )

        settings = DocViewerSettings(self.context)
        image_format = settings.pdf_image_format or 'gif'

        paths = []
        for page_num in range(1, 5):
            for size in ('small', 'normal', 'large'):
                paths.append(base_image_path + '/{}/dump_{}.{}'.format(
                    size, page_num, image_format))

        return paths

    def getAbsolutePaths(self):
        return []



from Acquisition import aq_inner
from castle.cms import utils
from castle.cms.interfaces import ICastleLayer
from castle.cms.interfaces import IGlobalTile
from castle.cms.interfaces import IMetaTile
from castle.cms.interfaces import IVersionViewLayer
from plone import api
from plone.tiles.interfaces import ITile
from plone.tiles.interfaces import ITileDataContext
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


@implementer(ITileDataContext)
@adapter(Interface, ICastleLayer, IGlobalTile)
def globalTileDataContext(context, request, tile):
    """
    Make sure to not pull from drafts data proxy
    """
    return tile.context


@implementer(ITileDataContext)
@adapter(Interface, ICastleLayer, IMetaTile)
def metaTileDataContext(context, request, tile):
    """
    Make sure to not pull from drafts data proxy
    """
    dc = getattr(tile, 'data_context', None)
    return dc or tile.context


@implementer(ITileDataContext)
@adapter(Interface, IVersionViewLayer, ITile)
def versionTileDataContext(context, request, tile):
    """
    Get version of context.
    This is used when viewing different versions of content
    """
    version = request.form.get('version')
    return utils.get_object_version(tile.context, version)



from Products.CMFPlone.interfaces import INavigationSchema
from zope.component import getUtility
from plone.registry.interfaces import IRegistry
from Acquisition import aq_parent
from castle.cms.interfaces import IGlobalTile
from castle.cms.tiles.base import BaseTile
from castle.cms.tiles.views import BaseTileView
from castle.cms.tiles.views import getTileView
from castle.cms.tiles.views import TileViewsSource
from castle.cms.utils import parse_query_from_data
from castle.cms.widgets import NavigationTypeWidget
from castle.cms.widgets import QueryFieldWidget
from castle.cms.widgets import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.dexterity.interfaces import IDexterityContainer
from plone.memoize.instance import memoize
from plone.supermodel import model
from plone.uuid.interfaces import IUUID
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope import schema
from zope.interface import implements
from zope.schema.vocabulary import SimpleVocabulary


class NavigationTile(BaseTile):
    implements(IGlobalTile)

    def render(self):
        view = getTileView(self.context, self.request, 'navigation',
                           self.data.get('display_type', 'horizontal') or 'horizontal',
                           default='horizontal')
        if view is None:
            view = HorizontalView(self.context, self.request)

        view.tile = self
        return view()

    def get_label(self, brain):
        try:
            nav_label = brain.navigation_label
            if nav_label:
                return nav_label
        except:
            pass
        return brain.Title

    @property
    def query(self):
        parsed = parse_query_from_data(self.data, self.context)
        # XXX we're forcing location queries to be depth of 1
        if 'path' in parsed:
            parsed['path']['depth'] = 1
        return parsed

    def items(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        query = None
        nav_type = self.data.get('nav_type', 'currentlocation') or 'currentlocation'
        if nav_type in ('query', 'secondlevel', 'currentlocation'):
            if nav_type == 'query':
                query = self.query
            elif nav_type == 'secondlevel':
                section = self.get_section()
                if section is not None:
                    query = self.build_nav_query({
                        'path': {
                            'depth': 1,
                            'query': '/'.join(section.getPhysicalPath())
                        }
                    })
            elif nav_type == 'currentlocation':
                context = self.context
                if hasattr(self, 'data_context'):
                    context = self.data_context
                query = self.build_nav_query({
                    'path': {
                        'depth': 1,
                        'query': '/'.join(context.getPhysicalPath())
                    }
                })
            if query is None:
                return []
            limit = self.data.get('limit') or 10
            items = catalog(**query)[:limit + 40]
            return [i for i in items if i.exclude_from_nav is not True][:limit]
        else:
            uids = self.data.get('content') or []
            results = dict([(b.UID, b) for b in catalog(UID=uids)])
            items = []
            for uid in uids:
                if uid in results:
                    items.append(results[uid])
            return items

    def build_nav_query(self, base_query):
        query = {}

        registry = getUtility(IRegistry)
        navigation_settings = registry.forInterface(
            INavigationSchema,
            prefix="plone"
        )
        query = {
            'sort_on': navigation_settings.sort_tabs_on,
            'portal_type': navigation_settings.displayed_types
        }

        if navigation_settings.sort_tabs_reversed:
            query['sort_order'] = 'reverse'

        if navigation_settings.filter_on_workflow:
            query['review_state'] = navigation_settings.workflow_states_to_show

        query.update(base_query)
        return query

    @memoize
    def get_section(self):
        section = None
        context = self.context
        while not IPloneSiteRoot.providedBy(context):
            section = context
            context = aq_parent(context)
        if section is not None and IDexterityContainer.providedBy(section):
            return section

    @memoize
    def get_parent_uids(self):
        uids = []
        context = self.context
        while not IPloneSiteRoot.providedBy(context):
            uids.append(IUUID(context, None))
            context = aq_parent(context)
        return [uid for uid in uids if uid is not None]


class HorizontalView(BaseTileView):
    name = 'horizontal'
    order = 0
    index = ViewPageTemplateFile('templates/navigation/horizontal.pt')
    tile_name = 'navigation'


class VerticalView(BaseTileView):
    name = 'vertical'
    order = 10
    index = ViewPageTemplateFile('templates/navigation/vertical.pt')
    tile_name = 'navigation'


class INavigationTileSchema(model.Schema):

    display_type = schema.Choice(
        title=u"Display Type",
        source=TileViewsSource('navigation'),
        default='horizontal'
    )

    form.widget(content=RelatedItemsFieldWidget)
    content = schema.List(
        title=u"Navigation items",
        description=u"Select items for navigation",
        required=False,
        default=[],
        value_type=schema.Choice(
            vocabulary='plone.app.vocabularies.Catalog'
        )
    )

    form.widget(query=QueryFieldWidget)
    query = schema.List(
        title=u'Search terms',
        description=u"Define the search terms for the items you want "
                    u"to list by choosing what to match on. "
                    u"The list of results will be dynamically updated.",
        value_type=schema.Dict(value_type=schema.Field(),
                               key_type=schema.TextLine()),
        required=False
    )

    sort_on = schema.TextLine(
        title=u'Sort on',
        description=u"Sort on this index",
        required=False,
    )

    sort_reversed = schema.Bool(
        title=u'Reversed order',
        description=u'Sort the results in reversed order',
        required=False,
    )

    form.widget(nav_type=NavigationTypeWidget)
    nav_type = schema.Choice(
        title=u'Use dynamic query to populate this menu',
        description=u'Otherwise, the navigation will be built from site section',
        vocabulary=SimpleVocabulary([
            SimpleVocabulary.createTerm('query', 'query', u'Query'),
            SimpleVocabulary.createTerm('content', 'content', u'Select items'),
            SimpleVocabulary.createTerm('currentlocation', 'currentlocation', u'Build from here'),
            SimpleVocabulary.createTerm('secondlevel', 'secondlevel',
                                        u'Build from second level from site root'),
        ]),
        default=u'currentlocation')

    limit = schema.Int(
        title=u'Limit',
        description=u'Limited number of items',
        required=True,
        default=10
    )



from zope.interface import implementer
from zope.component.interfaces import ObjectEvent, IObjectEvent


class IMetaTileEditedEvent(IObjectEvent):
    pass


class MetaTileEvent(ObjectEvent):
    def __init__(self, object, tile_id, **descriptions):
        super(MetaTileEvent, self).__init__(object, **descriptions)
        self.tile_id = tile_id


@implementer(IMetaTileEditedEvent)
class MetaTileEditedEvent(MetaTileEvent):
    pass



from AccessControl.SecurityManagement import newSecurityManager
from BTrees.OOBTree import OOBTree
from castle.cms.services import twitter
from castle.cms.social import COUNT_ANNOTATION_KEY
from castle.cms.utils import index_in_es
from castle.cms.utils import retriable
from datetime import datetime
from plone.app.redirector.interfaces import IRedirectionStorage
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from Products.CMFPlone.log import logger
from requests_oauthlib import OAuth1
from tendo import singleton
from zope.annotation.interfaces import IAnnotations
from zope.component import getUtility
from zope.component.hooks import setSite

import json
import sys
import time
import transaction


def record_tweet_stats(site, ob, tweet):
    """
    Keep daily totals...
    Total score includes complete impression(so favorites)
    """

    site._p_jar.sync()
    annotations = IAnnotations(ob)
    social_data = annotations.get(COUNT_ANNOTATION_KEY, OOBTree())
    stats = social_data.get('twitter', {
        'total': 0
    })

    score = 1 + tweet['favorite_count']

    now = datetime.utcnow()
    if now.year not in stats:
        year = stats[now.year] = {}
    else:
        year = stats[now.year]
    if now.month not in year:
        month = year[now.month] = {}
    else:
        month = year[now.month]
    if now.day not in month:
        month[now.day] = score
    else:
        month[now.day] += score

    stats['total'] += score
    social_data['twitter'] = stats
    annotations[COUNT_ANNOTATION_KEY] = social_data
    transaction.commit()


@retriable(sync=True)
def parse_line(site, public_url, line):
    tweet = json.loads(line)
    modified = False
    for url in tweet['entities']['urls']:
        if url['expanded_url'].startswith(public_url):
            path = url['expanded_url'].replace(
                public_url, '').split('?')[0].strip('/')
            ob = None
            if path in ('', '/'):
                try:
                    ob = site[site.default_page]
                except:
                    pass
            else:
                ob = site.restrictedTraverse(path, None)
            if ob is None:
                redirector = getUtility(IRedirectionStorage)
                obj_path = redirector.get(path, default=None)
                if obj_path:
                    ob = site.restrictedTraverse(obj_path, None)
            if ob:
                modified = True
                record_tweet_stats(site, ob, tweet)
    if modified:
        transaction.commit()
        index_in_es()


class StreamListener(twitter.StreamListener):
    def __init__(self, site, public_url):
        self.site = site
        self.public_url = public_url

    def on_data(self, raw_data):
        try:
            parse_line(self.site, self.public_url, raw_data)
        except:
            logger.warn('error parsing tweet', exc_info=True)


def attempt_twitter_on_site(site):
    setSite(site)

    registry = getUtility(IRegistry)
    public_url = registry.get('plone.public_url', None)
    if not public_url:
        return

    auth = OAuth1(
        registry['plone.twitter_consumer_key'].encode('utf8'),
        client_secret=registry['plone.twitter_consumer_secret'].encode('utf8'),
        resource_owner_key=registry['plone.twitter_oauth_token'].encode('utf8'),
        resource_owner_secret=registry['plone.twitter_oauth_secret'].encode('utf8'),
        decoding=None)

    # normlize url...
    original_url = public_url = public_url.replace(
        'http://', '').replace('https://', '').strip('/')
    if public_url.count('.') > 1:
        _, _, public_url = public_url.partition('.')

    stream = twitter.Stream(auth, StreamListener(site, original_url))
    stream.filter(track=[' '.join(public_url.split('.'))])


def run(app):
    singleton.SingleInstance('twittermonitor')

    user = app.acl_users.getUser('admin')  # noqa
    newSecurityManager(None, user.__of__(app.acl_users))  # noqa

    while True:
        try:
            if 'site-id' in sys.argv:
                siteid = sys.argv['site-id']
                attempt_twitter_on_site(app[siteid])  # noqa
            else:
                for oid in app.objectIds():  # noqa
                    obj = app[oid]  # noqa
                    if IPloneSiteRoot.providedBy(obj):
                        attempt_twitter_on_site(obj)
        except KeyError:
            pass
        logger.info('Could not find valid site to monitor')
        time.sleep(10 * 60)


if __name__ == '__main__':
    run(app)  # noqa



from AccessControl.SecurityManagement import newSecurityManager
from castle.cms.utils import retriable
from DateTime import DateTime
from plone import api
from plone.app.linkintegrity.exceptions import LinkIntegrityNotificationException
from plone.locking.interfaces import ILockable
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from tendo import singleton
from zope.component.hooks import setSite

import transaction


@retriable(sync=True)
def empty(site):
    setSite(site)
    catalog = api.portal.get_tool('portal_catalog')
    end = DateTime() - 30
    query = dict(
        modified={
            'query': (DateTime('1999/09/09'), end),
            'range': 'min:max'
        },
        trashed=True)
    count = 0
    for brain in catalog(**query):
        count += 1
        ob = brain.getObject()
        lockable = ILockable(ob, None)
        if lockable and lockable.locked():
            lockable.clear_locks()
        try:
            api.content.delete(ob)
        except LinkIntegrityNotificationException:
            # quietly ignore for now
            pass
        if count % 20 == 0:
            transaction.commit()
    transaction.commit()


def run(app):
    singleton.SingleInstance('emptytrash')

    user = app.acl_users.getUser('admin')  # noqa
    newSecurityManager(None, user.__of__(app.acl_users))  # noqa

    for oid in app.objectIds():  # noqa
        obj = app[oid]  # noqa
        if IPloneSiteRoot.providedBy(obj):
            empty(obj)


if __name__ == '__main__':
    run(app)  # noqa



from AccessControl.SecurityManagement import newSecurityManager
from BTrees.OOBTree import OOBTree
from castle.cms.services import google
from castle.cms.social import COUNT_ANNOTATION_KEY
from castle.cms.utils import retriable
from collective.elasticsearch.es import ElasticSearchCatalog
from plone import api
from plone.app.layout.navigation.defaultpage import getDefaultPage
from plone.uuid.interfaces import IUUID
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from tendo import singleton
from zope.annotation.interfaces import IAnnotations
from zope.component.hooks import setSite

import transaction


def get_results(service, profile_id):
    # Use the Analytics Service Object to query the Core Reporting API
    # for the number of sessions within the past seven days.
    return service.data().ga().get(
        ids='ga:' + profile_id,
        start_date='30daysAgo',
        end_date='today',
        metrics='ga:pageviews',
        sort='-ga:pageviews',
        dimensions='ga:pagePath').execute()


@retriable(sync=True)
def get_popularity(site):
    setSite(site)
    catalog = api.portal.get_tool('portal_catalog')
    es = ElasticSearchCatalog(catalog)
    if not es.enabled:
        return

    service = google.get_ga_service()
    if not service:
        return

    profile = google.get_ga_profile(service)
    if not profile:
        return

    bulk_data = []
    bulk_size = es.get_setting('bulk_size', 50)
    conn = es.connection

    site._p_jar.sync()
    for path, page_views in get_results(service, profile)['rows']:
        path = path.split('?')[0].lstrip('/').replace('/view', '').split('@@')[0]
        ob = site.restrictedTraverse(str(path), None)
        if ob is None:
            continue

        annotations = IAnnotations(ob)
        data = {
            'page_views': int(page_views)
        }
        counts = annotations.get(COUNT_ANNOTATION_KEY, OOBTree())
        counts['page_views'] = int(page_views)
        annotations[COUNT_ANNOTATION_KEY] = counts
        for key, value in counts.items():
            if key in ('page_views',):
                continue
            data[key + '_shares'] = value

        if IPloneSiteRoot.providedBy(ob):
            ob = ob[getDefaultPage(ob)]

        bulk_data.extend([{
            'update': {
                '_index': es.index_name,
                '_type': es.doc_type,
                '_id': IUUID(ob)
            }
        }, {'doc': data}])

        if len(bulk_data) % bulk_size == 0:
            conn.bulk(index=es.index_name, doc_type=es.doc_type, body=bulk_data)
            bulk_data = []
            transaction.commit()
            site._p_jar.sync()

    if len(bulk_data) > 0:
        conn.bulk(index=es.index_name, doc_type=es.doc_type, body=bulk_data)
    transaction.commit()


def run(app):
    singleton.SingleInstance('popularity')

    user = app.acl_users.getUser('admin')  # noqa
    newSecurityManager(None, user.__of__(app.acl_users))  # noqa

    for oid in app.objectIds():  # noqa
        obj = app[oid]  # noqa
        if IPloneSiteRoot.providedBy(obj):
            get_popularity(obj)


if __name__ == '__main__':
    run(app)  # noqa



from Acquisition import aq_parent
from castle.cms import authentication
from castle.cms.testing import SELENIUM_PLONE_FUNCTIONAL_TESTING
from castle.cms.tests import utils as test_utils
from plone.app.testing import selenium_layers
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC  # noqa
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support.ui import Select

import os
import transaction
import unittest2 as unittest


_dir = os.path.dirname(os.path.realpath(__file__))
TEST_FILE = os.path.join(_dir, 'testimage.png')


class CreateSite(unittest.TestCase):
    """
    Read: https://pypi.python.org/pypi/plone.app.testing
          https://github.com/plone/plone.app.testing/blob/master/plone/app/testing/selenium_layers.pypi
    for more info

    Xvfb :99 &
    export DISPLAY=:99

    To save screenshot while debugging tests:
        self.selenium.save_screenshot('/opt/plone/src/foobar.png')
    """

    layer = SELENIUM_PLONE_FUNCTIONAL_TESTING
    username_selector = '.castle-secure-login-form-code-authorized .username'
    password_selector = '.castle-secure-login-form-code-authorized .password'
    login_selector = ('.castle-secure-login-form-code-authorized '
                      '.castle-secure-login-login-button')

    def setUp(self):
        self.selenium = self.layer['selenium']
        self.portal = self.layer['portal']
        self.app = aq_parent(self.portal)
        authentication.install_acl_users(self.app)
        transaction.commit()

    def login(self):
        self.open(self.app.absolute_url() + '/@@secure-login')
        self.get(self.username_selector).send_keys(SITE_OWNER_NAME)
        self.get(self.password_selector).send_keys(SITE_OWNER_PASSWORD)
        self.get(self.login_selector).click()
        self.selenium.get(self.portal.absolute_url())
        test_utils.dismiss_tour(self.selenium, 8)

    def open(self, url):
        selenium_layers.open(self.selenium, url)

    def click(self, selector):
        test_utils.click_it(self.selenium, By.CSS_SELECTOR, selector)

    def get(self, selector, detection=EC.visibility_of_element_located):
        return test_utils.get_element(
            self.selenium, By.CSS_SELECTOR, selector, detection=detection)

    def screenshot(self):
        filepath = os.path.join(test_utils.src_dir, 'current.png')
        self.selenium.get_screenshot_as_file(filepath)

    def wait_then_click(self, selector, wait=10):
        el = WebDriverWait(self.selenium, wait).until(
            EC.presence_of_element_located(
                (By.CSS_SELECTOR, selector)))
        el.click()
        return el

    def get_element(self, method, locator, wait=10):
        return test_utils.get_element(self.selenium, method, locator, wait)

    def is_element_present(self, method, locator, wait=10):
        try:
            WebDriverWait(self.selenium, wait).until(
                EC.presence_of_element_located((method, locator)))
            return True
        except:
            return False

    def test_query_listing(self):
        selenium = self.selenium
        self.login()
        # create a folder
        test_utils.create_folder(selenium, 'folder-query', 'testfolder')
        # add tag for folder
        test_utils.add_tag(selenium, 'test_tag')
        # create some content in this new folder
        # create a page
        test_utils.create_page(selenium, 'basic')
        # add tag for this page
        # click edit
        test_utils.add_tag(selenium, 'test_tag')
        # create a page
        test_utils.create_page(selenium, 'document')
        # add tag for this page
        test_utils.add_tag(selenium, 'test_tag')
        test_utils.create_page(selenium, 'twocolumn')
        test_utils.add_tag(selenium, 'test_tag')
        test_utils.create_page(selenium, 'threecolumn')
        test_utils.add_tag(selenium, 'test_tag')
        # fill rest with news items until pagenation occurs
        pagen_trigger = 12
        while pagen_trigger > 0:
            test_utils.create_news_item(selenium, 'test_news%s' % pagen_trigger)
            test_utils.add_tag(selenium, 'test_tag')
            pagen_trigger -= 1

        selenium.get(self.portal.absolute_url() + '/testfolder')

    def test_add_from_dashboard(self):
        selenium = self.selenium
        self.login()
        # navigate to dashboard
        selenium.get(self.portal.absolute_url() + '/@@dashboard')
        # add a page
        self.get('.add-content-btn.contenttype-document').click()
        # provid page title
        self.get('#contentTitle').send_keys('foobar-dash')
        # click save on modal dialog
        self.click('#add-modal-react-container .modal-footer .plone-btn-primary')
        self.click('.mosaic-button-save')
        self.assertEqual(
            self.selenium.current_url, self.portal.absolute_url() + '/foobar-dash')

    def test_image_edit(self):
        selenium = self.selenium
        self.login()
        selenium.get(self.portal.absolute_url())
        # get image for upload
        try:
            test_utils.prep_file_upload(self.selenium, TEST_FILE, 'image')
        except:
            test_utils.prep_file_upload(self.selenium, TEST_FILE, 'image')
        # click image edit button
        self.get('.file-list .castle-btn-edit').click()
        # click rotate
        self.get('.castle-cropper-container .castle-btn-rotate-left').click()
        # click save
        self.get('.castle-cropper-container .castle-btn-save').click()
        # click upload
        self.get('.file-list .castle-btn-upload').click()
        # will error if can't find it
        self.get('.finished-container')

    def test_tile_edit(self):
        selenium = self.selenium
        self.login()
        selenium.get(self.portal.absolute_url())
        # create basic page
        test_utils.create_page(selenium, 'basic')
        # click edit
        test_utils.click_it(selenium, By.CSS_SELECTOR, '.castle-toolbar-edit a')
        # get title element
        edit_element = self.get('.mosaic-IDublinCore-title-tile .mosaic-rich-text')
        edit_element.click()
        # clear the text from this field
        test_utils.clear_it(edit_element)
        test_utils.dismiss_tour(self.selenium)
        edit_element.send_keys('new title is right here!')
        # click the save button
        self.click('.mosaic-button-save')
        self.assertTrue('new title' in self.selenium.page_source)

    def test_add_tile(self):
        selenium = self.selenium
        self.login()
        selenium.get(self.portal.absolute_url())
        # create a basic page
        test_utils.create_page(selenium, 'basic')
        # click edit
        self.click('.castle-toolbar-edit a')
        # click customize
        self.click('.mosaic-button-layout')
        self.click('.mosaic-button-customizelayout')
        # click insert select2
        test_utils.mousedown(selenium, By.CSS_SELECTOR, '.mosaic-menu-insert .select2-chosen')
        # select text tile to insert
        test_utils.mousedown(
            selenium, By.CSS_SELECTOR,
            '.select2-results .mosaic-option-plone-app-standardtiles-rawhtml .select2-result-label')
        # click on the page to place this new text tile
        click_placement = self.get('#content .mosaic-IDublinCore-description-tile')
        actionchain = ActionChains(selenium)
        actionchain.move_to_element_with_offset(click_placement, 0, 0)
        actionchain.click(click_placement)
        actionchain.perform()

        container = self.get('.mosaic-selected-tile')
        container.find_element_by_class_name('mosaic-rich-text')

        # click the save button
        try:
            self.click('.mosaic-button-save')
        except:
            self.selenium.find_element_by_id('form-buttons-save').click()

    def test_image_insert(self):
        selenium = self.selenium
        self.login()
        selenium.get(self.portal.absolute_url())
        test_utils.create_page(selenium, 'document')
        # click edit
        self.click('.castle-toolbar-edit a')
        # click description area so it receives focus and mce toolbar is visible
        self.click('#content .mosaic-IRichText-text-tile .mosaic-rich-text')
        self.click('#content .mosaic-IDublinCore-description-tile')
        self.click('#content .mosaic-IRichText-text-tile .mosaic-rich-text')
        # click the mce insert image button
        self.get('[aria-label="Insert/edit image"] button').click()
        # click the external tab
        self.click("#tinymce-autotoc-autotoc-1")
        # specify the external image url
        self.get(".form-group input[name='externalImage']").send_keys(
            'https://plone.org/logo@2x.png')
        # give the image a title
        self.get(".form-group input[name='alt']").send_keys('testimage')
        # click the insert button
        self.get('div.pattern-modal-buttons > input[name="insert"]').click()
        # click the save button
        test_utils.click_it(selenium, By.CSS_SELECTOR, '.mosaic-button-save')

    def test_add_video(self):
        selenium = self.selenium
        self.login()
        selenium.get(self.portal.absolute_url())
        test_utils.create_page(selenium, 'basic')
        self.get('.castle-toolbar-edit a').click()
        test_utils.mousedown(selenium, By.CSS_SELECTOR, '.mosaic-button-layout')
        test_utils.mousedown(selenium, By.CSS_SELECTOR, '.mosaic-button-customizelayout')
        test_utils.mousedown(selenium, By.CSS_SELECTOR, '.mosaic-menu-insert .select2-choice')
        test_utils.mousedown(selenium, By.CSS_SELECTOR, '.mosaic-option-castle-cms-videotile')
        tmpelement = self.get('#castle-cms-videotile-youtube_url')
        tmpelement.send_keys('https://www.youtube.com/watch?v=dQw4w9WgXcQ')
        test_utils.click_it(selenium, By.CSS_SELECTOR, '.plone-modal-footer #buttons-save')
        test_utils.click_it(selenium, By.CSS_SELECTOR, '#content > div:nth-child(2)')
        test_utils.mousedown(selenium, By.CSS_SELECTOR, '.mosaic-button-save')
        self.assertTrue(self.is_element_present(By.CSS_SELECTOR, '.video-container'))

    def test_change_page_title(self):
        selenium = self.selenium
        self.login()
        selenium.get(self.portal.absolute_url())
        # create the page
        test_utils.create_page(selenium, 'basic')
        # page is created, now edit the title
        self.get('.castle-toolbar-edit a').click()
        self.get('h1').send_keys('new title')
        self.click('.mosaic-button-save')

    def test_image_upload(self):
        self.login()
        self.selenium.get(self.portal.absolute_url())
        try:
            test_utils.prep_file_upload(self.selenium, TEST_FILE, 'image')
        except:
            test_utils.prep_file_upload(self.selenium, TEST_FILE, 'image')
        # click submit
        self.click('.castle-upload-container .castle-btn-upload')
        # will error if can't find it
        self.get('.finished-container')

    def test_file_upload(self):
        self.login()
        self.selenium.get(self.portal.absolute_url())
        test_utils.prep_file_upload(self.selenium, TEST_FILE, 'file')
        # click submit
        self.click('.castle-upload-container .castle-btn-upload')
        # will error if can't find it
        self.get('.finished-container')

    def test_createpages(self):
        selenium = self.selenium
        self.login()
        selenium.get(self.portal.absolute_url())
        test_utils.create_page(selenium, 'basic')
        self.assertEqual(self.selenium.current_url,
                         self.portal.absolute_url() + '/test_basic')
        selenium.get(self.portal.absolute_url())
        test_utils.create_page(selenium, 'document')
        self.assertEqual(self.selenium.current_url,
                         self.portal.absolute_url() + '/test_document')
        selenium.get(self.portal.absolute_url())
        test_utils.create_page(selenium, 'twocolumn')
        self.assertEqual(self.selenium.current_url,
                         self.portal.absolute_url() + '/test_twocolumn')
        selenium.get(self.portal.absolute_url())
        test_utils.create_page(selenium, 'threecolumn')
        self.assertEqual(self.selenium.current_url,
                         self.portal.absolute_url() + '/test_threecolumn')

    def test_create_page_under_folder(self):
        selenium = self.selenium
        self.login()
        selenium.get(self.portal.absolute_url())
        test_utils.create_folder(selenium, 'folder')
        test_utils.create_page(selenium, 'basic')
        self.assertEqual(self.selenium.current_url,
                         self.portal.absolute_url() + '/test_folder/test_basic')

    def test_createfolders(self):
        selenium = self.selenium
        self.login()
        selenium.get(self.portal.absolute_url())
        test_utils.create_folder(selenium, 'folder')
        self.assertEqual(self.selenium.current_url, self.portal.absolute_url() + '/test_folder')
        selenium.get(self.portal.absolute_url())
        test_utils.create_folder(selenium, 'folder-query')
        self.assertEqual(
            self.selenium.current_url, self.portal.absolute_url() + '/test_folder-query')

    def test_link(self):
        selenium = self.selenium
        self.login()
        selenium.get(self.portal.absolute_url())
        self.get('.castle-toolbar-add a').click()
        self.get('.contenttype-link-container a').click()
        self.get('#contentTitle').send_keys('foobarlink')
        test_utils.click_it(
            selenium, By.CSS_SELECTOR,
            '#add-modal-react-container .modal-footer .plone-btn-primary')
        test_utils.click_it(selenium, By.ID, 'form-buttons-save')
        self.assertEqual(self.selenium.current_url,
                         self.portal.absolute_url() + '/foobarlink')

    def test_event(self):
        self.login()
        self.generic_add('event')
        self.assertEqual(self.selenium.current_url,
                         self.portal.absolute_url() + '/test_event')

    def test_newsitem(self):
        selenium = self.selenium
        self.login()
        self.get('.castle-toolbar-add a').click()
        self.get('.contenttype-news-item-container a').click()
        self.get('#contentTitle').send_keys('foobar-newsitem')
        test_utils.click_it(
            selenium, By.CSS_SELECTOR,
            '#add-modal-react-container .modal-footer .plone-btn-primary')
        test_utils.click_it(self.selenium, By.CSS_SELECTOR, '.mosaic-button-save')
        self.assertEqual(self.selenium.current_url,
                         self.portal.absolute_url() + '/foobar-newsitem')

    def generic_add(self, testtype):
        self.get('.castle-toolbar-add a').click()
        self.get('.contenttype-%s-container a' % testtype).click()
        self.get('#contentTitle').send_keys('test_%s' % testtype)
        test_utils.click_it(
            self.selenium, By.CSS_SELECTOR,
            '#add-modal-react-container .modal-footer .plone-btn-primary')
        try:
            self.click('.mosaic-button-save')
        except:
            self.selenium.find_element_by_id('form-buttons-save').click()

    def test_impersonate(self):
        self.login()
        self.click('.icon-cog.plone-btn')
        self.click('.castle-toolbar-impersonate')
        self.click('.castle-btn-asanonymous')
        self.get('.userrole-anonymous')
        self.get('#impersonator')
        self.assertTrue('role="toolbar"' not in self.selenium.page_source)
        self.click('#impersonator a.stop')
        self.get('.castle-toolbar-container')

    def test_add_slot_tile(self):
        self.login()
        self.get('.castle-toolbar-slots a').click()
        self.get('.slot-chooser iframe')  # make sure it's loaded...
        self.selenium.switch_to_frame('choose_slot')
        self.get('[data-tile-id="meta-left"]').click()
        self.selenium.switch_to_default_content()
        self.get('[name="slotMode"][value="add"]').click()

        select = Select(self.get('.slot-manager-render-here select'))
        select.select_by_visible_text('Rich text')

        self.selenium.switch_to_frame(
            self.get('#plone-app-standardtiles-rawhtml-content_ifr'))
        self.get('.mce-content-body').send_keys('foobar slot')
        self.selenium.switch_to_default_content()

        # might be outside visibility of selenium screen...
        save_btn = self.get(
            '.pattern-modal-buttons input[value="Save"][type="submit"]',
            detection=EC.presence_of_element_located)
        save_btn.click()

        self.get('.modal .plone-btn').click()
        el = self.get('.tile-container-plone-app-standardtiles-rawhtml')
        self.assertTrue('foobar slot' in el.text)

    def test_preview(self):
        self.login()
        self.get('.castle-toolbar-preview a').click()

        select = Select(self.get('.castle-preview select'))
        select.select_by_visible_text('Tablet')

        self.assertEqual(self.get('.castle-preview iframe').size['width'], 800)

    def test_search(self):
        self.login()
        self.generic_add('document')
        self.open(self.portal.absolute_url() + '/@@search')

        self.get('.search-input').send_keys('test_document')
        self.assertTrue('test_document' in self.get('.searchResults li .result-title').text)



from castle.cms import cache
from castle.cms.utils import get_context_from_request
from plone import api
from Products.statusmessages.adapter import StatusMessage
from zope.i18n import translate

import time


class CastleStatusMessage(StatusMessage):
    max_messages = 10

    def __init__(self, context):  # should be request
        self.anon = api.user.is_anonymous()
        self.site_path = api.portal.get().getPhysicalPath()
        super(CastleStatusMessage, self).__init__(context)

    def get_cache_key(self):
        return '{}-{}-status-messages'.format(
            '-'.join(self.site_path[1:]),
            api.user.get_current().getId()
        )

    def add(self, text, type=u'info'):
        if self.anon:
            return super(CastleStatusMessage, self).add(text, type)

        try:
            text = translate(text)
        except:
            pass

        cache_key = self.get_cache_key()
        try:
            messages = cache.get(cache_key)
        except KeyError:
            messages = []

        site_path = context_path = '/'.join(self.site_path)
        context = get_context_from_request(self.context)
        if context:
            try:
                context_path = '/'.join(context.getPhysicalPath())
            except AttributeError:
                pass

        messages.append({
            'text': text,
            'type': type,
            'timestamp': time.time(),
            'context': context_path[len(site_path):]
        })
        messages = messages[-self.max_messages:]

        # cache for 1 hour, should it be longer? shorter?
        cache.set(cache_key, messages, 1 * 60 * 60)

    addStatusMessage = add

    def get_all(self):
        cache_key = self.get_cache_key()

        try:
            messages = cache.get(cache_key)
        except KeyError:
            messages = []
        return messages



# -*- coding: utf-8 -*-

import os

from setuptools import setup
from setuptools import find_packages


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()


setup(
    name='castle.cms',
    version='2.0.14',
    description='Castle CMS Plone distribution main package',
    long_description='%s\n%s' % (
        read('README.rst'),
        read('HISTORY.rst')
    ),
    keywords="plone cms castle",
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Framework :: Plone :: 5.0",
        "Framework :: Plone",
        "Framework :: Plone :: 5.1"
    ],
    author='Nathan Van Gheem',
    author_email='nathan.vangheem@wildcardcorp.com',
    url='https://github.com/castlecms/castle.cms',
    license='GPL2',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['castle'],
    install_requires=[
        'Products.CMFPlone',
        'Products.CMFPlacefulWorkflow',
        'plone.app.caching',
        'plone.app.dexterity',
        'plone.app.iterate',
        'plone.app.upgrade',
        'castle.theme',
        'setuptools',
        'collective.elasticsearch',
        'collective.celery',
        'requests',
        'requests_oauthlib',
        'plone.app.mosaic',
        'plone.api',
        'redis',
        'z3c.unconfigure',
        'collective.documentviewer',
        'python-dateutil',
        'boto',
        'google-api-python-client',
        'pyopenssl',
        'phonenumbers',
        'html2text',
        'z3c.jbot',
        'pycountry',
        'tendo',
        'plone.app.blocks>=10.0.0'
    ],
    extras_require={
        'test': [
            'selenium',
            'plone.app.testing',
            'responses',
            'mock',
            'argon2_cffi'
        ],
        'development': [
            'zest.releaser',
            'check-manifest',
        ],
        'profile': [
            'collective.profiler'
        ]
    },
    entry_points="""
      # -*- Entry points: -*-

      [z3c.autoinclude.plugin]
      target = plone

      [celery_tasks]
      castle = castle.cms.tasks

      [console_scripts]
      clean-plone-users = castle.cms.cron:clean_users
      social-counts = castle.cms.cron:social_counts
      archive-content = castle.cms.cron:archive_content
      content-popularity = castle.cms.cron:ga_popularity
      empty-trash = castle.cms.cron:empty_trash
      twitter-monitor = castle.cms.cron:twitter_monitor
      reindex-elasticsearch = castle.cms.cron:reindex_es
      send-forced-publish-alert = castle.cms.cron:forced_publish_alert
      castle-crawler = castle.cms.cron:crawler
      clean-drafts = castle.cms.cron:clean_drafts
      """,
    include_package_data=True,
    zip_safe=False,
)



from AccessControl import getSecurityManager
from Acquisition import aq_inner
from Acquisition import aq_parent
from castle.cms import caching
from castle.cms.browser.chat import get_chat_info
from castle.cms.interfaces import IDashboard
from castle.cms.interfaces import IToolbarModifier
from plone import api
from plone.app.blocks.layoutbehavior import ILayoutAware
from plone.app.layout.navigation.defaultpage import getDefaultPage
from plone.cachepurging.interfaces import ICachePurgingSettings
from plone.dexterity.interfaces import IDexterityContainer
from plone.dexterity.interfaces import IDexterityItem
from plone.i18n.normalizer.interfaces import IIDNormalizer
from plone.locking.interfaces import ITTWLockable
from plone.memoize.view import memoize
from plone.protect.authenticator import createToken
from plone.registry.interfaces import IRegistry
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import _checkPermission
from Products.CMFPlacefulWorkflow.permissions import ManageWorkflowPolicies
from Products.CMFPlone.interfaces.constrains import ISelectableConstrainTypes
from Products.CMFPlone.PloneBaseTool import createExprContext
from Products.Five import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from zope.component import getAllUtilitiesRegisteredFor
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryUtility
from zope.interface import implements

import copy
import json
import logging


logger = logging.getLogger('castle.cms')


class BaseToolbarModifier(object):
    order = 0
    layer = None
    implements(IToolbarModifier)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, result_menu, name, view):
        raise NotImplemented()


class MenuItem(object):

    _id = None
    title = None
    order = 0
    permission = None
    path = ''
    icon_class = None
    csrf = False
    condition = None
    attrs = {}

    def __init__(self, toolbar):
        self.toolbar = toolbar
        self.context = self.toolbar.real_context
        self.folder = self.toolbar.folder
        self.site = self.toolbar.site
        if self._id is None:
            self._id = self.title.replace(' ', '-').lower()
        if self.icon_class is None:
            self.icon_class = 'icon-' + self._id

    @property
    def url(self):
        url = self.context.absolute_url() + '/' + self.path
        if self.csrf:
            if '?' in url:
                url += '&'
            else:
                url += '?'
            return url + '_authenticator=' + self.toolbar.csrf_token
        else:
            return url

    @property
    def available(self):
        if self.permission is not None:
            if not api.user.has_permission(self.permission, obj=self.context):
                return False
        if self.condition is not None:
            try:
                if not self.condition(self):
                    return False
            except:
                logger.warn(
                    'Error running condition check for toolbar item %s' % self._id,
                    exc_info=True)
                return False
        return True

    def dict(self):
        attrs = self.attrs.copy()
        attrs.update({
            'title': self.title,
            'url': self.url,
            'id': self._id,
            'icon_class': self.icon_class
        })
        return attrs


class FolderMenuItem(MenuItem):
    def __init__(self, toolbar):
        super(FolderMenuItem, self).__init__(toolbar)
        if IDexterityItem.providedBy(self.context):
            self.context = aq_parent(self.context)


class SiteMenuItem(MenuItem):
    def __init__(self, toolbar):
        super(SiteMenuItem, self).__init__(toolbar)
        self.context = toolbar.site


class NonRootMenuItem(MenuItem):
    @property
    def available(self):
        if self.toolbar.context_state.is_portal_root():
            return False
        return super(NonRootMenuItem, self).available


Spacer = 'spacer'


def MenuItemFactory(title, Base=MenuItem, **kwargs):
    class _MenuItem(Base):
        def __init__(self, toolbar):
            self.title = title
            for field_name, value in kwargs.items():
                setattr(self, field_name, value)
            super(_MenuItem, self).__init__(toolbar)
    return _MenuItem


class Utils(object):

    @property
    @memoize
    def toolbar(self):
        toolbar = Toolbar(self.context, self.request)
        toolbar.update()
        return toolbar

    def user_folder(self):
        pm = api.portal.get_tool('portal_membership')
        return pm.getHomeUrl()

    def paste_available(self):
        if not IDexterityContainer.providedBy(self.toolbar.real_context):
            return False
        try:
            return self.toolbar.folder.cb_dataValid()
        except AttributeError:
            return True
        return False

    def non_root_item(self):
        menuItem = MenuItemFactory('#', Base=NonRootMenuItem)
        return menuItem(self.toolbar).available

    def container_url(self):
        context = self.context
        if IDexterityItem.providedBy(context):
            context = aq_parent(context)

        return context.absolute_url()

    def has_layout(self):
        return ILayoutAware.providedBy(self.context)

    def show_rename(self):
        return (api.user.has_permission("Delete objects", obj=self.toolbar.folder) and
                api.user.has_permission("Copy or Move", obj=self.toolbar.context) and
                api.user.has_permission("Add portal content", obj=self.toolbar.context) and not
                self.toolbar.context_state.is_portal_root())

    def show_invalidate(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ICachePurgingSettings, check=False)
        cf = caching.cloudflare.get()
        return cf.enabled or settings.cachingProxies


class Toolbar(BrowserView):
    """
    info this needs to gather...:

    - (x)menu items
    - (x)breadcrumbs
    - (x)available content to add
    - (x)user information
    - (x)site setup menu
    - (x)user menu
    - (x)workflow state information
    - (x)byline information
    - (x)locking information needs to be moved here as well
    """

    def get_addable_types(self):
        """Return menu item entries in a TAL-friendly form."""
        data = {
            'types': []
        }
        idnormalizer = queryUtility(IIDNormalizer)

        constraints = ISelectableConstrainTypes(self.folder, None)
        data['canConstrainTypes'] = False
        if constraints is not None:
            if constraints.canSetConstrainTypes() and \
                    constraints.getDefaultAddableTypes():
                data.update({
                    'canConstrainTypes': True,
                    'constrainUrl': '%s/folder_constraintypes_form' % (
                        self.folder.absolute_url(),)
                })

        site_path = '/'.join(self.site.getPhysicalPath())
        context = self.real_context
        if not IDexterityContainer.providedBy(context):
            context = aq_parent(context)
        folder_path = '/'.join(context.getPhysicalPath())[len(site_path):]
        if not folder_path:
            folder_path = '/'

        for t in self.folder.allowedContentTypes():
            typeId = t.getId()
            data['types'].append({
                'id': typeId,
                'safeId': idnormalizer.normalize(typeId),
                'title': t.Title(),
                'description': t.Description(),
                'folderPath': folder_path
            })

        return data

    def get_user_info(self):
        return {
            'id': self.user.getUserName(),
            'name': self.user.getProperty('fullname') or self.user.getUserName(),
            'last_login_time': self._toIso(self.user.getProperty('last_login_time'))
        }

    def get_workflow_info(self):
        state = state_id = None
        wf_tool = api.portal.get_tool('portal_workflow')
        workflows = wf_tool.getWorkflowsFor(self.real_context)
        if not workflows:
            return
        state_id = wf_tool.getInfoFor(ob=self.real_context, name='review_state')
        if not state_id:
            return
        for w in workflows:
            if state_id in w.states:
                state = w.states[state_id]
                break
        if not state:
            return

        transitions = []

        for action in wf_tool.listActionInfos(object=self.real_context):
            if action['category'] != 'workflow':
                continue

            transition = action.get('transition', None)
            if transition is not None:
                description = transition.description

            if action['allowed']:
                transitions.append({
                    'id': action['id'],
                    'title': action['title'],
                    'description': description
                })

        placeful = False
        try:
            pw = api.portal.get_tool('portal_placeful_workflow')
        except:
            pw = None
        if pw is not None:
            if _checkPermission(ManageWorkflowPolicies, self.real_context):
                placeful = True

        return {
            'state': {
                'id': state_id,
                'title': state.title,
                'description': state.description
            },
            'transitions': transitions,
            'placefulAllowed': placeful,
            'publishRequireQualityCheck': self.registry.get(
                'castle.publish_require_quality_check', True)
        }

    def _toIso(self, dt):
        if dt:
            return dt.ISO8601()

    def get_content_info(self):
        creator_id = self.real_context.Creator()
        creator = api.user.get(creator_id)
        contributors = []
        for contributor in self.real_context.Contributors():
            user = api.user.get(contributor)
            if user:
                contributors.append({
                    'id': contributor,
                    'name': user.getProperty('fullname') or contributor
                })
        return {
            'creator': {
                'id': creator_id,
                'name': creator and creator.getProperty('fullname') or creator_id
            },
            'expires': self._toIso(self.real_context.expires()),
            'effective': self._toIso(self.real_context.effective()),
            'modified': self._toIso(self.real_context.modified()),
            'contributors': contributors
        }

    def get_lock_info(self):
        if ITTWLockable.providedBy(self.real_context):
            info = getMultiAdapter((self.real_context, self.request), name="plone_lock_info")
            details = info.lock_info()
            if details:
                details = details.copy()
                _type = details.pop('type')
                details['timeout'] = _type.timeout
                details['user_unlockable'] = _type.user_unlockable
            return {
                'locked': info.is_locked_for_current_user(),
                'stealable': info.lock_is_stealable(),
                'details': details
            }

    @memoize
    def get_menu_modifiers(self):
        results = []
        utils = getAllUtilitiesRegisteredFor(IToolbarModifier)
        utils.sort(key=lambda u: u.order)
        for util in reversed(utils):
            if util.layer is not None:
                if not util.layer.providedBy(self.request):
                    continue
            results.append(util(self.real_context, self.request))
        return results

    def get_menu(self, name, additional=[]):
        menu = self._get_actions_menu(name, additional)
        for modifier in self.get_menu_modifiers():
            replacement = modifier(menu, name, self)
            if replacement:
                menu = replacement
        return menu

    def _get_portal_actions(self, name):
        if name not in self.pactions:
            return []
        return self.pactions[name].listActions()

    def has_permission(self, permissions):
        if type(permissions) not in (list, tuple, set):
            permissions = [permissions]
        sm = getSecurityManager()
        for permission in permissions:
            if sm.checkPermission(permission, self.real_context):
                return True
        return False

    def _get_actions_menu(self, name, additional=[]):
        menu = []

        actions = [x for x in self._get_portal_actions(name)]

        if additional is not []:
            # Include actions from the old default portal_actions to provide
            # support for Plone 5 add-ons
            for action in additional:
                actions += [x for x in self._get_portal_actions(action)]

        urlExpr = 'url_expr_object'
        conditionExpr = 'available_expr_object'
        iconExpr = 'icon_expr_object'

        for action in actions:
            if action.title == 'Spacer':
                menu.append('spacer')
                continue

            obj = copy.copy(action.__dict__)

            if obj.get('icon_class') is None:
                obj['icon_class'] = 'icon-%s' % obj['id']

            if urlExpr in obj:
                obj['url'] = obj[urlExpr](self.econtext)
                del obj[urlExpr]

            if iconExpr in obj:
                del obj[iconExpr]

            if conditionExpr in obj:
                if not obj[conditionExpr](self.econtext):
                    continue
                del obj[conditionExpr]

            if obj['permissions'] and not self.has_permission(obj['permissions']):
                continue

            if 'url' not in obj:
                obj['url'] = '#'

            if 'csrf' in obj:
                url = obj['url']
                if '?' in url:
                    url += '&'
                else:
                    url += '?'
                url = url + '_authenticator=' + self.csrf_token

                obj['url'] = url

            del obj['url_expr']

            menu.append(obj)

        return menu

    @property
    @memoize
    def real_context(self):
        context = self.context
        if ISiteRoot.providedBy(context):
            # we're at site root but actually kind of want context front page
            try:
                context = context[getDefaultPage(context)]
            except (AttributeError, KeyError):
                pass
        if IDashboard.providedBy(context):
            context = self.site
        return context

    def update(self):
        self.site = api.portal.get()

        self.context_state = getMultiAdapter(
            (aq_inner(self.real_context), self.request),
            name=u'plone_context_state')
        self.folder = self.context_state.folder()

        self.user = api.user.get_current()
        self.csrf_token = createToken()
        self.registry = getUtility(IRegistry)
        self.pactions = api.portal.get_tool('portal_actions')

        self.econtext = createExprContext(self.folder, self.site, self.real_context)

    def __call__(self):
        self.update()

        if (self.request.form.get('fc-reload') or
                'folder_contents' in self.request.URL or
                self.request.get('disable_border', False)):
            main_menu = self.get_menu('folder_contents_menu')
        else:
            main_menu = self.get_menu('toolbar_menu',
                                      ['object_buttons', 'object'])

        breadcrumbs_view = getMultiAdapter((self.real_context, self.request),
                                           name='breadcrumbs_view')

        user_menu = self.get_menu('user_menu', ['user'])
        management_menu = self.get_menu('management_menu')

        chat_info = get_chat_info()

        data = {
            'data-base-url': self.real_context.absolute_url(),
            'data-view-url': self.context_state.view_url(),
            'main_menu': main_menu,
            'management_menu': management_menu,
            'user_menu': user_menu,
            'add': self.get_addable_types(),
            'breadcrumbs': breadcrumbs_view.breadcrumbs(),
            'user': self.get_user_info(),
            'workflow': self.get_workflow_info(),
            'content': self.get_content_info(),
            'lock_info': self.get_lock_info(),
            'messages': list(reversed(IStatusMessage(self.request).get_all())),
            'user_id': api.user.get_current().getId(),
            'chat_info': chat_info
        }

        if '@@castle-toolbar' in self.request.URL:
            # if this was called from a request, set the ct header
            self.request.response.setHeader('Content-Type', 'application/json')

        return json.dumps(data)



from castle.cms.widgets import SelectFieldWidget
from Products.CMFPlone.controlpanel.browser import dateandtime


class DateAndTimeControlPanelForm(dateandtime.DateAndTimeControlPanelForm):

    def updateFields(self):
        super(DateAndTimeControlPanelForm, self).updateFields()
        self.fields['available_timezones'].widgetFactory = SelectFieldWidget


class DateAndTimeControlPanel(dateandtime.DateAndTimeControlPanel):
    form = DateAndTimeControlPanelForm



# -*- coding: utf-8 -*-
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone import api
from plone.app.testing import login
from plone.app.testing import setRoles
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING
from castle.cms import toolbar

import unittest
import json


class TestToolbar(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))
        api.content.create(
            type='Folder',
            id='folder',
            container=self.portal)

    def test_toolbar_renders_items(self):
        tlb = toolbar.Toolbar(self.portal, self.request)
        result = json.loads(tlb())
        self.assertTrue('main_menu' in result)
        self.assertEqual(result['main_menu'][0]['title'], 'View Page')

    def test_addable_types(self):
        tlb = toolbar.Toolbar(self.portal.folder, self.request)
        result = json.loads(tlb())
        self.assertTrue('add' in result)

    def test_breadcrumbs(self):
        tlb = toolbar.Toolbar(self.portal.folder, self.request)
        result = json.loads(tlb())
        self.assertEqual(result['breadcrumbs'][-1]['Title'], 'folder')



# -*- coding: utf-8 -*-
from castle.cms.behaviors.location import ILocation
from castle.cms.tiles.metadata import MetaDataTile
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING
from plone import api
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME

import unittest


class TestMetadata(unittest.TestCase):
    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))

    def test_gives_location(self):
        page = api.content.create(type='Document', title='Foobar', container=self.portal)
        ldata = ILocation(page)
        ldata.locations = [u'Green Bay, WI']

        tile = MetaDataTile(page, self.request)
        result = tile()
        self.assertTrue('Green Bay, WI' in result)
        self.assertTrue('name="location"' in result)



from zope.interface import Attribute
from zope.interface import Interface


class IToolbarModifier(Interface):
    layer = Attribute('layer to be used with')
    order = Attribute('order should be run with')

    def __call__(menu):
        pass



from plone.namedfile.file import NamedBlobImage
from unidecode import unidecode
from AccessControl.SecurityManagement import newSecurityManager
from lxml.html import fromstring
from lxml.html import tostring
from plone import api
from plone.app.contenttypes.behaviors.richtext import IRichText
from plone.app.textfield.value import RichTextValue
from plone.i18n.normalizer.interfaces import IIDNormalizer
from zope.component import getUtility
from zope.component.hooks import setSite

import argparse
import random
import re
import requests
import transaction


url_regex = re.compile(
    r'^(?:http)s?://'
    r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'
    r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'
    r'(?::\d+)?'
    r'(?:/?|[/?]\S+)$', re.IGNORECASE)


parser = argparse.ArgumentParser(
    description='...')
parser.add_argument('--file', dest='file', default=False)
parser.add_argument('--site-id', dest='site_id', default='Plone')
parser.add_argument('--limit', dest='limit', default=20, type=int)
args, _ = parser.parse_known_args()

user = app.acl_users.getUser('admin')  # noqa
newSecurityManager(None, user.__of__(app.acl_users))  # noqa
site = app[args.site_id]  # noqa
setSite(site)

container = site

normalizer = getUtility(IIDNormalizer)

resp = requests.get('https://en.wikipedia.org/wiki/Main_Page')
base_dom = dom = fromstring(resp.content)
parsed = []

while len(parsed) < args.limit:
    # find a link, parse it
    found_url = None
    while found_url is None:
        anchors = dom.cssselect('#bodyContent a')
        if len(anchors) == 0:
            anchors = base_dom.cssselect('#bodyContent a')
        anchor = random.choice(anchors)
        url = anchor.attrib.get('href')
        if (url.startswith('//') or 'Template:' in url or 'Wikipedia:' in url or
                'Category:' in url or 'index.php' in url or 'File:' in url or
                'Help:' in url or 'Portal:' in url or 'Talk:' in url):
            continue
        if url and url[0] == '/':
            url = 'https://en.wikipedia.org' + url
        if (url and url_regex.match(url) and url not in parsed and
                url.startswith('https://en.wikipedia.org')):
            found_url = url

    # randomly decide to make new folder to put stuff in
    if random.randint(0, 100) > 90:  # 10% chance
        container = api.content.create(
            type='Folder', title='Folder', container=container, exclude_from_nav=True)

    print('parsing ' + found_url)
    resp = requests.get(found_url)
    dom = fromstring(resp.content)
    try:
        title = unidecode(dom.cssselect('h1')[0].text_content())
    except IndexError:
        print('bad url: ' + found_url)
        continue
    _id = normalizer.normalize(found_url.split('/')[-1])
    if _id in container.objectIds():
        continue
    kw = {}
    imgs = dom.cssselect('#mw-content-text img')
    if len(imgs) > 0:
        im_url = imgs[0].attrib['src']
        if im_url.startswith('//'):
            im_url = 'https:' + im_url
        resp = requests.get(im_url)
        kw['image'] = NamedBlobImage(
            data=resp.content,
            filename=unidecode(im_url.split('/')[-1]).decode('utf8'))
    obj = api.content.create(type='Document', id=_id, title=title,
                             exclude_from_nav=True, container=container, **kw)
    text = ''
    for p in dom.cssselect('#mw-content-text p'):
        text += tostring(p)
    bdata = IRichText(obj, None)
    bdata.text = RichTextValue(text, 'text/html', 'text/html')

    obj.reindexObject()

    parsed.append(found_url)

transaction.commit()



from castle.cms.constants import CRAWLED_SITE_ES_DOC_TYPE
from castle.cms.utils import get_public_url
from collective.elasticsearch.es import ElasticSearchCatalog
from collective.elasticsearch.interfaces import IQueryAssembler
from DateTime import DateTime
from elasticsearch import TransportError
from plone import api
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import _getAuthenticatedUser
from Products.Five import BrowserView
from urlparse import urljoin
from urlparse import urlparse
from zope.component import getMultiAdapter
from zope.component import getUtility

import datetime
import json
import Missing


def custom_json_handler(obj):
    if obj == Missing.Value:
        return None
    if type(obj) in (datetime.datetime, datetime.date):
        return obj.isoformat()
    if type(obj) == DateTime:
        return obj.ISO8601()
    return obj


def _one(val):
    if val and type(val) == list and len(val) == 1:
        val = val[0]
    return val


class Search(BrowserView):

    @property
    def options(self):
        search_types = [{
            'id': 'images',
            'label': 'Images',
            'query': {
                'portal_type': 'Image'
            }
        }, {
            'id': 'page',
            'label': 'Page',
            'query': {
                'portal_type': ['Document', 'Folder']
            }
        }]

        ptypes = api.portal.get_tool('portal_types')
        for type_id in ptypes.objectIds():
            if type_id in ('Link', 'Document', 'Folder'):
                continue
            _type = ptypes[type_id]
            if not _type.global_allow:
                continue
            search_types.append({
                'id': type_id.lower(),
                'label': _type.title,
                'query': {
                    'portal_type': type_id
                }
            })
        search_types.extend([{
            'id': 'video',
            'label': 'Video',
            'query': {
                'portal_type': 'Video'
            }
        }, {
            'id': 'audio',
            'label': 'Audio',
            'query': {
                'portal_type': 'Audio'
            }
        }])

        additional_sites = []
        es = ElasticSearchCatalog(api.portal.get_tool('portal_catalog'))
        if es.enabled:
            query = {
                "size": 0,
                "aggregations": {
                    "totals": {
                        "terms": {
                            "field": "domain"
                        }
                    }
                }
            }
            try:
                result = es.connection.search(
                    index=es.index_name,
                    doc_type=CRAWLED_SITE_ES_DOC_TYPE,
                    body=query)
                for res in result['aggregations']['totals']['buckets']:
                    site_name = res.get('key')
                    if '.' not in site_name or 'amazon' in site_name:
                        continue
                    additional_sites.append(site_name)
            except TransportError:
                return []

        parsed = urlparse(get_public_url())

        return json.dumps({
            'searchTypes': search_types,
            'additionalSites': [s for s in sorted(additional_sites)],
            'currentSiteLabel': parsed.netloc
        })

    @property
    def search_url(self):
        if api.user.is_anonymous():
            try:
                url = api.portal.get_registry_record('castle.searchurl')
                if url:
                    return url
            except:
                pass
        return '%s/@@searchajax' % (
            self.context.absolute_url()
        )


_search_attributes = [
    'Title',
    'Description',
    'Subject',
    'contentType',
    'created',
    'modified',
    'effective',
    'hasImage',
    'is_folderish',
    'portal_type',
    'review_state',
    'url'
]
_valid_params = [
    'SearchableText',
    'portal_type',
    'Subject'
]


class SearchAjax(BrowserView):

    def __call__(self):
        self.catalog = api.portal.get_tool('portal_catalog')
        self.request.response.setHeader('Content-type', 'application/json')

        query = {}
        for name in _valid_params:
            if self.request.form.get(name):
                query[name] = self.request.form[name]
            elif self.request.form.get(name + '[]'):
                query[name] = self.request.form[name + '[]']

        try:
            page_size = int(self.request.form.get('pageSize'))
        except:
            page_size = 20
        page_size = min(page_size, 50)
        try:
            page = int(self.request.form.get('page'))
        except:
            page = 1
        catalog = api.portal.get_tool('portal_catalog')
        es = ElasticSearchCatalog(catalog)
        if es.enabled:
            return self.get_es_results(page, page_size, query)
        else:
            return self.get_results(page, page_size, query)

    def get_results(self, page, page_size, query):
        # regular plone search
        site_path = '/'.join(self.context.getPhysicalPath())
        start = (page - 1) * page_size
        end = start + page_size
        catalog = api.portal.get_tool('portal_catalog')
        raw_results = catalog(**query)
        items = []

        registry = getUtility(IRegistry)
        view_types = registry.get('plone.types_use_view_action_in_listings', [])

        for brain in raw_results[start:end]:
            attrs = {}
            for key in _search_attributes:
                attrs[key] = getattr(brain, key, None)
            url = base_url = brain.getURL()
            if brain.portal_type in view_types:
                url += '/view'
            attrs.update({
                'path': brain.getPath()[len(site_path):],
                'base_url': base_url,
                'url': url
            })
            items.append(attrs)

        return json.dumps({
            'count': len(raw_results),
            'results': items,
            'page': page,
            'suggestions': []
        }, default=custom_json_handler)

    def get_es_results(self, page, page_size, query):
        start = (page - 1) * page_size
        site_path = '/'.join(self.context.getPhysicalPath())
        base_site_url = self.context.absolute_url()

        results = suggestions = []
        count = 0
        if len(query) > 0:
            results = self.search_es(query, start, page_size)
            count = results['hits']['total']
            try:
                suggestions = results['suggest']['SearchableText'][0]['options']
            except:
                suggestions = []
            results = results['hits']['hits']

        registry = getUtility(IRegistry)
        view_types = registry.get('plone.types_use_view_action_in_listings', [])

        items = []
        for res in results:
            fields = res['fields']

            attrs = {}
            for key in _search_attributes:
                val = fields.get(key)
                attrs[key] = _one(val)

            if 'url' not in fields:
                path = fields.get('path.path', [''])
                path = path[0]
                path = path[len(site_path):]
                url = base_url = urljoin(base_site_url + '/', path.lstrip('/'))
                if attrs.get('portal_type') in view_types:
                    url += '/view'
            else:
                url = base_url = _one(fields['url'])
                parsed = urlparse(url)
                path = parsed.path

            attrs.update({
                'review_state': 'published',
                'score': res['_score'],
                'path': path,
                'base_url': base_url,
                'url': url
            })

            items.append(attrs)

        return json.dumps({
            'count': count,
            'results': items,
            'page': page,
            'suggestions': suggestions
        }, default=custom_json_handler)

    def search_es(self, query, start, size):
        user = _getAuthenticatedUser(self.catalog)
        query['allowedRolesAndUsers'] = self.catalog._listAllowedRolesAndUsers(user)

        es = ElasticSearchCatalog(self.catalog)
        qassembler = getMultiAdapter((self.request, es), IQueryAssembler)
        dquery, sort = qassembler.normalize(query)
        equery = qassembler(dquery)

        doc_type = es.doc_type
        if 'searchSite' in self.request.form:
            doc_type = CRAWLED_SITE_ES_DOC_TYPE
            equery = {
                'filtered': {
                    'filter': {
                        "term": {
                            "domain": self.request.form['searchSite']
                        }
                    },
                    'query': equery['function_score']['query']['filtered']['query']
                }
            }

        query = {
            'query': equery,
            "suggest": {
                "SearchableText": {
                    "text": query.get('SearchableText', ''),
                    "term": {
                        "field": "SearchableText"
                    }
                }
            }
        }

        query_params = {
            'from_': start,
            'size': size,
            'fields': ','.join(_search_attributes) + ',path.path',
        }

        return es.connection.search(index=es.index_name,
                                    doc_type=doc_type,
                                    body=query,
                                    **query_params)



# -*- coding: utf-8 -*-
from castle.cms.cron._crawler import Crawler
from castle.cms.interfaces import ICrawlerConfiguration
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING
from collective.elasticsearch.es import ElasticSearchCatalog
from plone import api
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
import responses

import unittest


class TestCrawl(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

    @responses.activate
    def test_crawl_page(self):
        responses.add(responses.GET, "https://www.foobar.com",
                      body=TEST_ARCHIVE_PAGE,
                      content_type="text/html")

        catalog = api.portal.get_tool('portal_catalog')
        es = ElasticSearchCatalog(catalog)
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ICrawlerConfiguration, prefix='castle')
        crawler = Crawler(self.portal, settings, es)
        data = crawler.crawl_page(
            'https://www.foobar.com')

        self.assertEquals(data['domain'], 'www.foobar.com')
        self.assertEquals(
            data['url'],
            'https://www.foobar.com')
        self.assertEquals(data['portal_type'], 'Form Folder')
        self.assertTrue(bool(data['Title']))
        self.assertTrue(bool(data['SearchableText']))


TEST_ARCHIVE_PAGE = '''

<!DOCTYPE html PUBLIC
                "-//W3C//DTD XHTML 1.0 Transitional//EN"
                "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
<meta http-equiv="Content-Type" content="text/html;charset=utf-8"/>
<meta name="generator" content="Plone - http://plone.org"/>


<!--[if IE]>
      <meta http-equiv="X-UA-Compatible" content="IE=8; IE=7; IE=5" />
      <![endif]-->
<base href="https://www.foobar.com/"/><!--[if lt IE 7]></base><![endif]-->
<meta name="DC.format" content="text/plain"/>
<meta name="DC.type" content="Form Folder"/>
<meta name="DC.date.valid_range" content="2012/02/09 - "/>
<meta name="DC.date.created" content="2016-06-20T14:12:16+00:00"/>
<meta name="DC.language" content="en"/>
<meta name="DC.date.modified" content="2016-06-20T14:53:04+00:00"/>
<style type="text/css" media="screen">@import url(https://www.foobar.com/portal_css/foobarTheme/base-cachekey-dbdb8fad070a40dcb5f536fa03ca1f1b.css);</style>
<style type="text/css" media="screen">@import url(https://www.foobar.com/portal_css/foobarTheme/columns-cachekey-e8b80fdb0d7fda3d97c9b4c121264879.css);</style>
<link rel="stylesheet" type="text/css" href="https://www.foobar.com/portal_css/foobarTheme/resourcetinymce.stylesheetstinymce-cachekey-0f418c220786988a7bf86e000fb1655b.css" media="screen"/>
<style type="text/css">@import url(https://www.foobar.com/portal_css/foobarTheme/print-cachekey-32662b10c7e3296c50f5776b4cd91e5f.css);</style>
<link rel="stylesheet" type="text/css" href="https://www.foobar.com/portal_css/foobarTheme/resourceplone.app.discussion.stylesheetsdiscussion-cachekey-bd4ed6e7272a04edb5a6bc1c1f5067b5.css" media="screen"/>
<!--[if IE 8]>

    <style type="text/css" media="screen">@import url(https://www.foobar.com/portal_css/foobarTheme/ie8fixes-cachekey-9213db1b6a69c06fa22b79ee009598b4.css);</style>
        <![endif]-->
<style type="text/css" media="screen">@import url(https://www.foobar.com/portal_css/foobarTheme/resourceeasyslider-portlet-cachekey-030b6fd1f2ea40e4a71ce418d774b5e0.css);</style>
<!--[if IE]>

    <style type="text/css">@import url(https://www.foobar.com/portal_css/foobarTheme/IEAllVersions-cachekey-c5721d6b43b43de3af1c0ec99daa1add.css);</style>
        <![endif]-->
<style type="text/css">@import url(https://www.foobar.com/portal_css/foobarTheme/jscalendarcalendar-system-cachekey-1e12e89e18a4d6b3d925ce836442f63a.css);</style>
<script type="text/javascript" src="https://www.foobar.com/portal_javascripts/foobarTheme/resourceplone.app.jquery-cachekey-f5487f1b1c781ef75c72f204a746c834.js"></script>
<script type="text/javascript" src="https://www.foobar.com/portal_javascripts/foobarTheme/resourceeasyslider-portlet-cachekey-c6142033b322f3b1acfae3b9269a431f.js"></script>
<script type="text/javascript" src="https://www.foobar.com/portal_javascripts/foobarTheme/foreseeforesee-alive-cachekey-b631c890f4c17b38353a07e694aa8d18.js"></script>
<script type="text/javascript" src="https://www.foobar.com/portal_javascripts/foobarTheme/custom-cachekey-07d8ddee05772bddba96179480d9b837.js"></script>
<title>Foobar</title>

<meta name="modificationDate" content="2016/06/20"/>
<meta name="creationDate" content="2016/06/20"/>
<meta name="publicationDate" content="2012/02/09"/>
<meta name="expirationDate"/>
<meta name="portalType" content="FormFolder"/>
<meta name="mobileEnabled"/>
<meta name="location" content=""/>
<meta name="contentType" content="text/plain"/>

<meta property="og:type" content="article"/>
<meta property="og:url" content="https://www.foobar.com"/>
<meta property="og:description" content=""/>
<meta name="DC.format" content="text/plain"/>
<meta name="DC.type" content="Form Folder"/>
<meta name="DC.date.valid_range" content="2012/02/09 - "/>
<meta name="DC.date.created" content="2016-06-20T14:12:16+00:00"/>
<meta name="DC.language" content="en"/>
<meta name="DC.date.modified" content="2016-06-20T14:53:04+00:00"/>
<link rel="canonical" href="https://www.foobar.com"/>
<link rel="shortcut icon" type="image/x-icon" href="https://www.foobar.com/favicon.ico"/>
<link rel="apple-touch-icon" href="https://www.foobar.com/touch_icon.png"/>

<meta http-equiv="imagetoolbar" content="no"/>
<script type="text/javascript" charset="iso-8859-1" src="https://www.foobar.com/widgets/js/textcount.js">
      </script>
<style type="text/css" media="screen">@import url(norfolk-speaker-request-form.css);</style>
</head>
<body class="obj-d48ed00212f8438eb4beb4b16bb0547b portaltype-formfolder template-fg_base_view_p3 portaltype-formfolder site-forms icons-on userrole-anonymous" dir="ltr">
<div id="shadow">
<div id="visual-portal-wrapper">
<div id="portal-top">
<div id="portal-header">
<div id="portal-searchbox">
<div id="portal-advanced-search" class="hiddenStructure">
<a href="https://www.foobar.com/search_form" accesskey="5">Advanced Search&hellip;</a>
</div>
</div>
<div id="portal-logo">
<a accesskey="1" href="https://www.foobar.com">
<img id="logo1" alt="Federal Bureau of Investigation" src="https://www.foobar.com/logo1.png"/><img id="logo2" alt="Federal Bureau of Investigation" src="https://www.foobar.com/logo2.png"/><img id="logo3" alt="Federal Bureau of Investigation" src="https://www.foobar.com/logo3.png"/>
</a>
</div>
<h5 class="hiddenStructure">Sections</h5>
<div class="visualClear"></div>
<div id="topnav-container">
</div>
<ul id="portal-globalnav"></ul>
</div>
<div id="portal-header-cell">
<div class="header-cell image-cell main-layout" colspan="2">
<img alt="Header Cell Image" src="https://www.foobar.com/header.jpg"/>
</div>
<div id="portal-breadcrumbs">
<a href="https://www.foobar.com"><font color="#b71318">Home</font></a>
<span class="breadcrumbSeparator">
&bull;
</span>
<span dir="ltr">
</span>
</div>
<div class="visualClear" id="clear-space-before-wrapper-table"> </div>
<table id="portal-columns">
<tbody>
<tr>
<td id="portal-column-content">
<div class="content_border_wrapper">
<div class="">
<div id="region-content" class="documentContent">
<span id="contentTopLeft"></span>
<span id="contentTopRight"></span>
<a name="documentContent"></a>
<dl class="portalMessage info" id="kssPortalMessage" style="display:none">
<dt>Info</dt>
<dd></dd>
</dl>
<div id="viewlet-above-content"></div>
<div id="content">
<div class="" id="parent-fieldname-formPrologue-d48ed00212f8438eb4beb4b16bb0547b">
<p align="center">Please note: Requests for speakers must be received by the Kansas City Field Office a minimum of 45 days <br/>prior to your presentation date. All requests are subject to availability.<strong> <br/></strong></p>
</div>
<div class="pfg-form">
<form name="edit_form" method="post" enctype="multipart/form-data" class="fgBaseEditForm enableUnloadProtection enableAutoFocus" action="https://www.foobar.com" id="fg-base-edit">
<div></div>
<div id="pfg-fieldwrapper">
<fieldset class="PFGFieldsetWidget" id="pfg-fieldsetname-column1">
<div class="formHelp" id="column1_help"></div>
<div class="field PFG-RichLabel" id="archetypes-fieldname-contact-information">
<p><strong><font size="3">Contact Information</font></strong></p>
</div>
<div class="field ArchetypesStringWidget " id="archetypes-fieldname-contact-person" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="contact-person">
<span></span>
<label class="formQuestion" for="contact-person">
Contact Person:
<span class="required" title="Required">&nbsp;</span>
<span class="formHelp" id="contact-person_help"></span>
</label>
<div class="fieldErrorBox"></div>
<input type="text" name="contact-person" class="blurrable firstToFocus" id="contact-person" size="30" maxlength="255"/>
</div>
<div class="field ArchetypesStringWidget " id="archetypes-fieldname-position-title" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="position-title">
<span></span>
<label class="formQuestion" for="position-title">
Position/Title:
<span class="required" title="Required">&nbsp;</span>
<span class="formHelp" id="position-title_help"></span>
</label>
<div class="fieldErrorBox"></div>
<input type="text" name="position-title" class="blurrable firstToFocus" id="position-title" size="30" maxlength="255"/>
</div>
<div class="field ArchetypesStringWidget " id="archetypes-fieldname-contacts-email-address" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="contacts-email-address">
<span></span>
<label class="formQuestion" for="contacts-email-address">
Contact's E-Mail Address:
<span class="required" title="Required">&nbsp;</span>
<span class="formHelp" id="contacts-email-address_help"></span>
</label>
<div class="fieldErrorBox"></div>
<input type="text" name="contacts-email-address" class="blurrable firstToFocus" id="contacts-email-address" size="30" maxlength="255"/>
</div>
<div class="field ArchetypesStringWidget " id="archetypes-fieldname-contacts-phone" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="contacts-phone">
<span></span>
<label class="formQuestion" for="contacts-phone">
Contact's Phone:
<span class="required" title="Required">&nbsp;</span>
<span class="formHelp" id="contacts-phone_help"></span>
</label>
<div class="fieldErrorBox"></div>
<input type="text" name="contacts-phone" class="blurrable firstToFocus" id="contacts-phone" size="30" maxlength="255"/>
</div>
<div class="field ArchetypesStringWidget " id="archetypes-fieldname-contacts-secondary-phone" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="contacts-secondary-phone">
<span></span>
<label class="formQuestion" for="contacts-secondary-phone">
Contact's Secondary Phone:
<span class="formHelp" id="contacts-secondary-phone_help"></span>
</label>
<div class="fieldErrorBox"></div>
<input type="text" name="contacts-secondary-phone" class="blurrable firstToFocus" id="contacts-secondary-phone" size="30" maxlength="255"/>
</div>
<div class="field PFG-RichLabel" id="archetypes-fieldname-about-the-organization">
<p><strong><font size="3">About the Organization<br/></font></strong></p>
</div>
<div class="field ArchetypesStringWidget " id="archetypes-fieldname-organization-group-name" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="organization-group-name">
<span></span>
<label class="formQuestion" for="organization-group-name">
Organization/Group Name
<span class="required" title="Required">&nbsp;</span>
<span class="formHelp" id="organization-group-name_help"></span>
</label>
<div class="fieldErrorBox"></div>
<input type="text" name="organization-group-name" class="blurrable firstToFocus" id="organization-group-name" size="30" maxlength="255"/>
</div>
<div class="field ArchetypesSelectionWidget " id="archetypes-fieldname-audience-type" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="audience-type">
<span></span>
<div class="fieldErrorBox"></div>
<span id="audience-type">

<div class="formQuestion label">
Audience Type
<span class="required" title="Required">&nbsp;</span>
<span class="formHelp" id="audience-type_help"></span>
</div>
<input class="noborder blurrable" type="radio" name="audience-type" id="audience-type_1" value="Business Professionals"/>
<label for="audience-type_1">Business Professionals</label>
<br/>
<input class="noborder blurrable" type="radio" name="audience-type" id="audience-type_2" value="Civic Group"/>
<label for="audience-type_2">Civic Group</label>
<br/>
<input class="noborder blurrable" type="radio" name="audience-type" id="audience-type_3" value="Faith-Based Organization"/>
<label for="audience-type_3">Faith-Based Organization</label>
<br/>
<input class="noborder blurrable" type="radio" name="audience-type" id="audience-type_4" value="Law Enforcement"/>
<label for="audience-type_4">Law Enforcement</label>
<br/>
<input class="noborder blurrable" type="radio" name="audience-type" id="audience-type_5" value="Seniors"/>
<label for="audience-type_5">Seniors</label>
<br/>
<input class="noborder blurrable" type="radio" name="audience-type" id="audience-type_6" value="Teens"/>
<label for="audience-type_6">Teens</label>
<br/>
<input class="noborder blurrable" type="radio" name="audience-type" id="audience-type_7" value="Other (describe below in comments section)"/>
<label for="audience-type_7">Other (describe below in comments section)</label>
<br/>
</span>
</div>
<div class="field PFG-RichLabel" id="archetypes-fieldname-presentation-date-location">
<p><strong><font size="3">Presentation Date/Location</font></strong></p>
</div>
<div class="field ArchetypesStringWidget " id="archetypes-fieldname-event-date-time" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="event-date-time">
<span></span>
<label class="formQuestion" for="event-date-time">
Event Date/Time:
<span class="required" title="Required">&nbsp;</span>
<span class="formHelp" id="event-date-time_help"></span>
</label>
<div class="fieldErrorBox"></div>
<input type="text" name="event-date-time" class="blurrable firstToFocus" id="event-date-time" size="30" maxlength="255"/>
</div>
<div class="field ArchetypesTextAreaWidget " id="archetypes-fieldname-event-location" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="event-location">
<span></span>
<label class="formQuestion" for="event-location">
Event Location:
<span class="required" title="Required">&nbsp;</span>
<span class="formHelp" id="event-location_help"></span>
</label>
<div class="fieldErrorBox"></div>
<textarea class="blurrable firstToFocus" name="event-location" id="event-location" cols="40" rows="5"></textarea>
</div>
</fieldset>
<fieldset class="PFGFieldsetWidget" id="pfg-fieldsetname-column2">
<div class="formHelp" id="column2_help"></div>
<div class="field PFG-RichLabel" id="archetypes-fieldname-about-the-event"><strong><font size="3">About the Event<br/></font></strong></div>
<div class="field ArchetypesTextAreaWidget " id="archetypes-fieldname-purpose-of-the-event" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="purpose-of-the-event">
<span></span>
<label class="formQuestion" for="purpose-of-the-event">
Purpose of the Event:
<span class="required" title="Required">&nbsp;</span>
<span class="formHelp" id="purpose-of-the-event_help"></span>
</label>
<div class="fieldErrorBox"></div>
<textarea class="blurrable firstToFocus" name="purpose-of-the-event" id="purpose-of-the-event" cols="40" rows="5"></textarea>
</div>
<div class="field ArchetypesStringWidget " id="archetypes-fieldname-number-of-attendees-expected" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="number-of-attendees-expected">
<span></span>
<label class="formQuestion" for="number-of-attendees-expected">
Number of Attendees Expected
<span class="required" title="Required">&nbsp;</span>
<span class="formHelp" id="number-of-attendees-expected_help">(minimum 25 attendees)</span>
</label>
<div class="fieldErrorBox"></div>
<input type="text" name="number-of-attendees-expected" class="blurrable firstToFocus" id="number-of-attendees-expected" size="30" maxlength="255"/>
</div>
<div class="field ArchetypesStringWidget " id="archetypes-fieldname-length-of-presentation-e.g.-30-45-minutes" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="length-of-presentation-e.g.-30-45-minutes">
<span></span>
<label class="formQuestion" for="length-of-presentation-e.g.-30-45-minutes">
Length of Presentation (e.g. 30-45 minutes)
<span class="required" title="Required">&nbsp;</span>
<span class="formHelp" id="length-of-presentation-e.g.-30-45-minutes_help"></span>
</label>
<div class="fieldErrorBox"></div>
<input type="text" name="length-of-presentation-e.g.-30-45-minutes" class="blurrable firstToFocus" id="length-of-presentation-e.g.-30-45-minutes" size="30" maxlength="255"/>
</div>
<div class="field PFG-RichLabel" id="archetypes-fieldname-presentation-topic">
<p><strong><font size="3">Presentation Topic<br/></font></strong></p>
</div>
<div class="field ArchetypesMultiSelectionWidget " id="archetypes-fieldname-choose-a-topic-that-suits-your-event-select-all-that-apply" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="choose-a-topic-that-suits-your-event-select-all-that-apply">
<span></span>
<div class="fieldErrorBox"></div>
<label class="formQuestion" for="choose-a-topic-that-suits-your-event-select-all-that-apply">
Choose a topic that suits your event (select all that apply)
<span class="required" title="Required">&nbsp;</span>
</label>
<select multiple="multiple" class="blurrable" name="choose-a-topic-that-suits-your-event-select-all-that-apply:list" id="choose-a-topic-that-suits-your-event-select-all-that-apply" size="5">
<option value="Terrorism">Terrorism</option>
<option value="CREST Program*">CREST Program*</option>
<option value="Cyber Crime">Cyber Crime</option>
<option value="Public Corruption">Public Corruption</option>
<option value="Gangs">Gangs</option>
<option value="Civil Rights">Civil Rights</option>
<option value="Recruitment and Hiring">Recruitment and Hiring</option>
<option value="Youth-Based Presentation">Youth-Based Presentation</option>
<option value="Other (describe below in comments section)">Other (describe below in comments section)</option>
</select>
</div>
<div class="field PFG-RichLabel" id="archetypes-fieldname-other-event-information">
<p><strong><font size="3">Other Event Information<br/></font></strong></p>
</div>
<div class="field ArchetypesSelectionWidget " id="archetypes-fieldname-will-there-be-q-a-afterwards" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="will-there-be-q-a-afterwards">
<span></span>
<div class="fieldErrorBox"></div>
<span id="will-there-be-q-a-afterwards">

<div class="formQuestion label">
Will there be Q&amp;A afterwards?
<span class="required" title="Required">&nbsp;</span>
<span class="formHelp" id="will-there-be-q-a-afterwards_help"></span>
</div>
<input class="noborder blurrable" type="radio" name="will-there-be-q-a-afterwards" id="will-there-be-q-a-afterwards_1" value="Yes"/>
<label for="will-there-be-q-a-afterwards_1">Yes</label>
<br/>
<input class="noborder blurrable" type="radio" name="will-there-be-q-a-afterwards" id="will-there-be-q-a-afterwards_2" value="No"/>
<label for="will-there-be-q-a-afterwards_2">No</label>
<br/>
</span>
</div>
<div class="field ArchetypesSelectionWidget " id="archetypes-fieldname-will-there-be-audio-visual-equipment-available-for-powerpoint-slides" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="will-there-be-audio-visual-equipment-available-for-powerpoint-slides">
<span></span>
<div class="fieldErrorBox"></div>
<span id="will-there-be-audio-visual-equipment-available-for-powerpoint-slides">

<div class="formQuestion label">
Will there be audio/visual equipment available for PowerPoint slides?
<span class="required" title="Required">&nbsp;</span>
<span class="formHelp" id="will-there-be-audio-visual-equipment-available-for-powerpoint-slides_help"></span>
</div>
<input class="noborder blurrable" type="radio" name="will-there-be-audio-visual-equipment-available-for-powerpoint-slides" id="will-there-be-audio-visual-equipment-available-for-powerpoint-slides_1" value="Yes"/>
<label for="will-there-be-audio-visual-equipment-available-for-powerpoint-slides_1">Yes</label>
<br/>
<input class="noborder blurrable" type="radio" name="will-there-be-audio-visual-equipment-available-for-powerpoint-slides" id="will-there-be-audio-visual-equipment-available-for-powerpoint-slides_2" value="No"/>
<label for="will-there-be-audio-visual-equipment-available-for-powerpoint-slides_2">No</label>
<br/>
</span>
</div>
<div class="field ArchetypesSelectionWidget " id="archetypes-fieldname-is-a-bio-needed-for-the-speaker" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="is-a-bio-needed-for-the-speaker">
<span></span>
<div class="fieldErrorBox"></div>
<span id="is-a-bio-needed-for-the-speaker">

<div class="formQuestion label">
Is a bio needed for the speaker?
<span class="required" title="Required">&nbsp;</span>
<span class="formHelp" id="is-a-bio-needed-for-the-speaker_help"></span>
</div>
<input class="noborder blurrable" type="radio" name="is-a-bio-needed-for-the-speaker" id="is-a-bio-needed-for-the-speaker_1" value="Yes"/>
<label for="is-a-bio-needed-for-the-speaker_1">Yes</label>
<br/>
<input class="noborder blurrable" type="radio" name="is-a-bio-needed-for-the-speaker" id="is-a-bio-needed-for-the-speaker_2" value="No"/>
<label for="is-a-bio-needed-for-the-speaker_2">No</label>
<br/>
</span>
</div>
<div class="field ArchetypesSelectionWidget " id="archetypes-fieldname-will-media-be-present" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="will-media-be-present">
<span></span>
<div class="fieldErrorBox"></div>
<span id="will-media-be-present">

<div class="formQuestion label">
Will media be present?
<span class="required" title="Required">&nbsp;</span>
<span class="formHelp" id="will-media-be-present_help"></span>
</div>
<input class="noborder blurrable" type="radio" name="will-media-be-present" id="will-media-be-present_1" value="Yes"/>
<label for="will-media-be-present_1">Yes</label>
<br/>
<input class="noborder blurrable" type="radio" name="will-media-be-present" id="will-media-be-present_2" value="No"/>
<label for="will-media-be-present_2">No</label>
<br/>
</span>
</div>
<div class="field ArchetypesSelectionWidget " id="archetypes-fieldname-do-you-plan-to-do-any-publicity-related-to-this-event-either-before-or-after-including-through-press-releases-websites-blog-posts-photos-or-other-means" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="do-you-plan-to-do-any-publicity-related-to-this-event-either-before-or-after-including-through-press-releases-websites-blog-posts-photos-or-other-means">
<span></span>
<div class="fieldErrorBox"></div>
<span id="do-you-plan-to-do-any-publicity-related-to-this-event-either-before-or-after-including-through-press-releases-websites-blog-posts-photos-or-other-means">

<div class="formQuestion label">
Do you plan to do any publicity related to this event (either before or after), including through press releases, websites, blog posts, photos, or other means?
<span class="required" title="Required">&nbsp;</span>
<span class="formHelp" id="do-you-plan-to-do-any-publicity-related-to-this-event-either-before-or-after-including-through-press-releases-websites-blog-posts-photos-or-other-means_help"></span>
</div>
<input class="noborder blurrable" type="radio" name="do-you-plan-to-do-any-publicity-related-to-this-event-either-before-or-after-including-through-press-releases-websites-blog-posts-photos-or-other-means" id="do-you-plan-to-do-any-publicity-related-to-this-event-either-before-or-after-including-through-press-releases-websites-blog-posts-photos-or-other-means_1" value="Yes"/>
<label for="do-you-plan-to-do-any-publicity-related-to-this-event-either-before-or-after-including-through-press-releases-websites-blog-posts-photos-or-other-means_1">Yes</label>
<br/>
<input class="noborder blurrable" type="radio" name="do-you-plan-to-do-any-publicity-related-to-this-event-either-before-or-after-including-through-press-releases-websites-blog-posts-photos-or-other-means" id="do-you-plan-to-do-any-publicity-related-to-this-event-either-before-or-after-including-through-press-releases-websites-blog-posts-photos-or-other-means_2" value="No"/>
<label for="do-you-plan-to-do-any-publicity-related-to-this-event-either-before-or-after-including-through-press-releases-websites-blog-posts-photos-or-other-means_2">No</label>
<br/>
</span>
</div>
<div class="field ArchetypesTextAreaWidget " id="archetypes-fieldname-if-yes-please-explain" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="if-yes-please-explain">
<span></span>
<label class="formQuestion" for="if-yes-please-explain">
If yes, please explain.
<span class="formHelp" id="if-yes-please-explain_help"></span>
</label>
<div class="fieldErrorBox"></div>
<textarea class="blurrable firstToFocus" name="if-yes-please-explain" id="if-yes-please-explain" cols="40" rows="5"></textarea>
</div>
</fieldset>
<div class="field ArchetypesTextAreaWidget " id="archetypes-fieldname-comments" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="comments">
<span></span>
<label class="formQuestion" for="comments">
Comments
<span class="formHelp" id="comments_help"></span>
</label>
<div class="fieldErrorBox"></div>
<textarea class="blurrable firstToFocus" name="comments" id="comments" cols="40" rows="5"></textarea>
</div>
<div class="field ArchetypesCaptchaWidget " id="archetypes-fieldname-captcha" data-uid="d48ed00212f8438eb4beb4b16bb0547b" data-fieldname="captcha">
<span></span>
<label class="formQuestion" for="captcha">
Captcha
<span class="required" title="Required">&nbsp;</span>
<span class="formHelp" id="captcha_help"></span>
</label>
<div class="fieldErrorBox"></div>
<script type="text/javascript">
        function playAudio(url) {
        document.getElementById("captcha-audio-embed").innerHTML=
        "<embed src='"+url+"' hidden=true autostart=true loop=false>";
        }
        </script>
<div class="captchaImage">
<script type="text/javascript">
    var RecaptchaOptions = {
        lang: 'en'
    };
    </script>
<script type="text/javascript" src="https://www.google.com/recaptcha/api/challenge?k=6LfDkLwSAAAAAD4IfxM2UpXo_jrotbWHVPWLwZVN&hl=en"></script>
<noscript>
<iframe src="https://www.google.com/recaptcha/api/noscript?k=6LfDkLwSAAAAAD4IfxM2UpXo_jrotbWHVPWLwZVN&hl=en" height="300" width="500" frameborder="0"></iframe><br/>
<textarea name="recaptcha_challenge_field" rows="3" cols="40"></textarea>
<input type='hidden' name='recaptcha_response_field' value='manual_challenge'/>
</noscript>
&nbsp;
</div>
<input type="hidden" name="captcha" value="x"/>
</div>
</div>
<div class="formControls">
<input type="hidden" name="fieldset" value="default"/>
<input type="hidden" name="form.submitted" value="1"/>
<input type="hidden" name="add_reference.field:record" value=""/>
<input type="hidden" name="add_reference.type:record" value=""/>
<input type="hidden" name="add_reference.destination:record" value=""/>
<input type="hidden" name="last_referer" value="https://www.foobar.com"/>
<input type="hidden" name="_authenticator" value="69f4453815e7831cf99ec3eb0c08bec1a6a2c36a"/>
<input class="context" type="submit" name="form_submit" value="Submit"/>
</div>
</form>
</div>
<script>
    // block inline validation
    jQuery(function ($) {
        $("#pfg-fieldwrapper .field").removeAttr('data-uid');
    });
</script>
</div>
<span id="contentBottomLeft"></span>
<span id="contentBottomRight"></span>
</div>
</div>
</td>
</tr>
</tbody>
</table>
</div>
</div>
<div class="visualClear" id="clear-space-before-footer"> </div>
<div id="portal-footer">
<div class="footerWrapper">
<div align="center">
</div>
</div>
</div>
<div id="portal-footer2">
<div class="footer2Wrapper">

</div>
<div id="kss-spinner"><img alt="" src="https://www.foobar.com/spinner.gif"/></div>
</div>
</body>
</html>
'''



from zope.component import getMultiAdapter
from castle.cms import utils
from collective.documentviewer.settings import GlobalSettings as DVGlobalSettings
from plone.app.blocks.layoutbehavior import ILayoutAware
from plone import api
from plone.registry.interfaces import IRegistry
from Products.CMFCore.WorkflowCore import WorkflowException
from Products.CMFPlone.interfaces.constrains import ISelectableConstrainTypes
from Products.CMFPlone.interfaces.syndication import ISiteSyndicationSettings
from zope.component import getUtility
from plone.tiles.interfaces import ITileDataManager


INDEXES = {
    "contentType": "KeywordIndex",
    "location": "KeywordIndex",
    "hasImage": "BooleanIndex",
    "trashed": "BooleanIndex"
}

REMOVE_INDEXES = [
    'in_reply_to',
    'sync_uid',
    'total_comments',
    'cmf_uid',
    'commentators',
    'is_default_page'
]

METADATA = [
    'hasImage',
    'contentType',
    'last_modified_by',
    'image_info',
    'navigation_label'
]

REMOVE_METADATA = [
    'cmf_uid',
    'commentators',
    'in_response_to',
    'last_comment_date',
    'sync_uid',
    'total_comments',
    'listCreators'
]


def _removeTinyMCEActions(data):
    for name in ('undo', 'redo', 'bold', 'italic'):
        if 'toolbar-' + name in data:
            data.remove('toolbar-' + name)


def castle(context):
    if not context.readDataFile('castle.cms.install.txt'):
        return
    site = context.getSite()

    # create feed folder
    folder = utils.recursive_create_path(site, '/feeds')
    try:
        if api.content.get_state(obj=folder) != 'published':
            api.content.transition(obj=folder, transition='publish')
    except WorkflowException:
        pass

    type_ = 'Collection'
    aspect = ISelectableConstrainTypes(folder, None)

    if (aspect and (
            aspect.getConstrainTypesMode() != 1 or
            [type_] != aspect.getImmediatelyAddableTypes())):
        aspect.setConstrainTypesMode(1)
        aspect.setImmediatelyAddableTypes([type_])
    if not getattr(folder, 'exclude_from_nav', False):
        folder.exclude_from_nav = True
        folder.reindexObject()

    if 'front-page' not in site:
        api.content.create(type='Document', id='front-page', container=site)
        site.setDefaultPage('front-page')

    front_page = site['front-page']

    front_page.title = u'Welcome to Castle'
    front_page.description = u'Congratulations! You have successfully installed Castle.'

    # enable syndication by default and modify some of the settings
    registry = getUtility(IRegistry)
    settings = registry.forInterface(ISiteSyndicationSettings)
    settings.allowed = True
    settings.default_enabled = False
    settings.show_author_info = False
    settings.search_rss_enabled = False

    utils.add_indexes(INDEXES)
    utils.delete_indexes(REMOVE_INDEXES)
    utils.add_metadata(METADATA)
    utils.delete_metadata(REMOVE_METADATA)

    # add some better defaults for documentviewer
    settings = DVGlobalSettings(site)
    settings.auto_layout_file_types = [
        'pdf', 'word', 'excel', 'ppt', 'rft', 'ps', 'photoshop', 'visio', 'palm']

    # delete some records for mosaic tinymce toolbar
    for action_type in ('plone_app_z3cform_wysiwyg_widget_WysiwygWidget',
                        'plone_app_z3cform_wysiwyg_widget_WysiwygFieldWidget',
                        'plone_app_widgets_dx_RichTextWidget',
                        'plone_app_z3cform_widget_RichTextFieldWidget'):
        try:
            data = registry['plone.app.mosaic.widget_actions.%s.actions' % action_type]
            _removeTinyMCEActions(data)
            registry['plone.app.mosaic.widget_actions.%s.actions' % action_type] = data
        except KeyError:
            pass

    for key in ('plone.app.mosaic.structure_tiles.text.available_actions',
                'plone.app.mosaic.app_tiles.plone_app_standardtiles_rawhtml.available_actions'):
        try:
            data = registry[key]
            _removeTinyMCEActions(data)
            registry[key] = data
        except KeyError:
            pass

    # password reset timeout interval...
    portal_password_reset = api.portal.get_tool('portal_password_reset')
    portal_password_reset.setExpirationTimeout(6)

    # update default session duration
    site.acl_users.session.timeout = 1 * 60 * 60  # 1 hr
    site.acl_users.session.refresh_interval = 5 * 60  # 5 minutes



from collections import namedtuple

SHIELD = namedtuple('SHIELD', 'NONE BACKEND ALL')('', 'backend', 'all')
MAX_PASTE_ITEMS = 40
ALL_SUBSCRIBERS = '--SUBSCRIBERS--'
ALL_USERS = '--USERS--'
DEFAULT_SITE_LAYOUT_REGISTRY_KEY = 'castle.cms.mosaic.default_site_layout'
CRAWLED_SITE_ES_DOC_TYPE = 'website'
CRAWLED_DATA_KEY = 'castle.cms.crawldata'



# -*- coding: utf-8 -*-
from castle.cms.files.duplicates import DuplicateDetector
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING
from plone import api
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.uuid.interfaces import IUUID

import unittest


class TestDuplicateDetector(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))

    def test_register(self):
        dd = DuplicateDetector()
        hash_ = 'foobar'
        doc = api.content.create(
            type='Document',
            id='folder',
            container=self.portal)
        dd.register(doc, hash_)

        self.assertEqual(IUUID(dd.get_object(hash_)), IUUID(doc))



from castle.cms.behaviors.location import ILocation
from castle.cms.widgets import SelectFieldWidget
from plone import api
from plone.app.z3cform.layout import wrap_form
from Products.CMFPlone.browser.syndication import adapters
from Products.CMFPlone.browser.syndication import settings
from Products.CMFPlone.browser.syndication import views
from Products.CMFPlone.browser.syndication.views import FeedView
from Products.CMFPlone.interfaces.syndication import IFeedSettings as IBaseFeedSettings
from Products.CMFPlone.interfaces.syndication import ISiteSyndicationSettings
from Products.CMFPlone.interfaces.syndication import ISyndicatable
from Products.Five import BrowserView
from z3c.form import button
from z3c.form import field
from zope import schema
from zope.component import adapts
from zope.component import getUtility
from zope.interface import implements
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleVocabulary

import json


class IFeedSettings(IBaseFeedSettings):
    sort_on = schema.Choice(
        title=u'Sort on',
        default='effective',
        vocabulary=SimpleVocabulary([
            SimpleVocabulary.createTerm('effective', 'effective', 'Publication Date'),
            SimpleVocabulary.createTerm('modified', 'modified', 'Modification Date'),
            SimpleVocabulary.createTerm('start', 'start', 'Start(event)'),
            SimpleVocabulary.createTerm('getObjPositionInParent',
                                        'getObjPositionInParent', 'Position')
        ])
    )

    sort_reverse = schema.Bool(
        title=u'Reverse sort',
        description=u'Order items in reverse order',
        default=True)

    categories = schema.List(
        title=u'ITunes categories',
        description=u'Only for itunes feed',
        value_type=schema.TextLine(),
        default=[],
        required=False,
        missing_value=[]
    )


class FeedSettings(settings.FeedSettings):
    implements(IFeedSettings)
    adapts(ISyndicatable)

    def __getattr__(self, name):
        default = None
        if name in ISiteSyndicationSettings.names():
            default = getattr(self.site_settings, name)
        elif name == 'enabled' and self.site_settings.default_enabled:
            default = True
        elif name in IFeedSettings.names():
            default = IFeedSettings[name].default

        return self._metadata.get(name, default)


class SettingsForm(views.SettingsForm):

    def update(self):
        if self.context.portal_type in ('Collection', 'Feed'):
            self.fields = field.Fields(IFeedSettings).omit('sort_on', 'sort_reverse')
        else:
            self.fields = field.Fields(IFeedSettings)
        self.fields['feed_types'].widgetFactory = SelectFieldWidget
        super(SettingsForm, self).update()

    @button.buttonAndHandler(u'Cancel', name='cancel')
    def handleCancel(self, action):
        self.request.response.redirect(self.context.absolute_url())

    @button.buttonAndHandler(u'Save', name='save')
    def handleSave(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        self.applyChanges(data)
        self.request.response.redirect(self.context.absolute_url())

SettingsFormView = wrap_form(SettingsForm)


class FolderFeed(adapters.FolderFeed):
    def _brains(self):
        catalog = api.portal.get_tool('portal_catalog')
        sort_order = 'ascending'
        if self.settings.sort_reverse or self.settings.sort_reverse is None:
            sort_order = 'reverse'
        return catalog(
            path={
                'query': '/'.join(self.context.getPhysicalPath()),
                'depth': 1
            },
            sort_order=sort_order,
            sort_on=self.settings.sort_on or 'effective',
            sort_limit=self.settings.limit or 15)


class KMLFeedView(FeedView):
    content_type = 'application/vnd.google-earth.kml+xml'

    def get_coordinates(self, item):
        data = ILocation(item.context, None)
        if data is None:
            return

        try:
            data = json.loads(data.coordinates)
            if type(data) == dict:
                data = [data]
            return data
        except:
            pass


class JSONFeedView(FeedView):
    content_type = 'application/json'

    def index(self):
        items = []
        feed = self.feed()
        for item in feed.items:
            items.append({
                'title': item.title,
                'description': item.description,
                'url': item.link,
                'modified': item.modified.ISO8601(),
                'file_url': item.file_type and item.file_url,
                'file_type': item.file_type,
                'file_length': item.file_length,
                'image_url': item.image_url,
                'uid': feed.uid
            })

        return json.dumps({
            'url': feed.link,
            'title': feed.title,
            'description': feed.description,
            'modified': feed.modified.ISO8601(),
            'uid': feed.uid,
            'logo': feed.logo,
            'items': items
        })


class ContentFeedView(BrowserView):

    @property
    def feed_types(self):
        factory = getUtility(IVocabularyFactory,
                             "plone.app.vocabularies.SyndicationFeedTypes")
        vocabulary = factory(self.context)
        types = []
        settings = IBaseFeedSettings(self.context)
        for typ in settings.feed_types:
            types.append(vocabulary.getTerm(typ))
        return types



from castle.cms.files import aws
from castle.cms.media import video
from castle.cms.utils import retriable
from collective.celery import task
from plone.registry.interfaces import IRegistry
from plone.uuid.interfaces import IUUID
from zope.annotation.interfaces import IAnnotations
from zope.component import getUtility

import transaction


def _clean_aws(obj):
    annotations = IAnnotations(obj)
    if aws.STORAGE_KEY in annotations and obj.file.filename != aws.FILENAME:
        del annotations[aws.STORAGE_KEY]
        aws.delete_file(IUUID(obj))


@task()
def file_edited(obj):
    _file_edited(obj)


@retriable()
def _file_edited(obj):
    try:
        if not obj.file:
            return _clean_aws(obj)
    except AttributeError:
        return
    if obj.portal_type == 'Video':
        video.process(obj)
        transaction.commit()

    if obj.portal_type not in ('Video', 'Audio', 'File'):
        return _clean_aws(obj)

    if 'pdf' in obj.file.contentType:
        # we also aren't moving pdfs out of here
        return _clean_aws(obj)

    registry = getUtility(IRegistry)
    if registry.get('castle.aws_s3_key', None) is None:
        return

    max_size_mb = registry.get('castle.max_file_size', 50)
    max_size = max_size_mb * 1024 * 1024
    if obj.file.getSize() > max_size:
        aws.move_file(obj)
    else:
        return _clean_aws(obj)


@task()
def workflow_updated(obj):
    annotations = IAnnotations(obj)
    if aws.STORAGE_KEY in annotations:
        aws.set_permission(obj)


@task()
def aws_file_deleted(uid):
    aws.delete_file(uid)



from castle.cms.tiles.base import BaseTile
from plone.autoform import directives as form
from plone.supermodel import model
from zope import schema
from plone import api
from castle.cms.widgets import RelatedItemFieldWidget
from zope.component import getMultiAdapter

from zope.interface import implements
from plone.tiles.interfaces import IPersistentTile


class DocumentViewerTile(BaseTile):
    implements(IPersistentTile)

    def __call__(self):
        if self.data.get('documentUID') is None:
            return ''

        doc = api.content.get(UID=self.data['documentUID'])
        if doc:
            viewer = getMultiAdapter((doc, self.request), name="docviewertile")
            return viewer()
        return '<html><body><pCould not find file</p></body></html>'


class IDocumentViewerTileSchema(model.Schema):
    form.widget(documentUID=RelatedItemFieldWidget)
    documentUID = schema.List(
        title=u'Document',
        description=u'The file to display in the document viewer',
        required=True
    )



from zope.interface import Interface


class IUtils(Interface):

    def get_registry_value(name, default):
        pass

    def get_object_url(object):
        pass

    def get_object(val):
        pass

    def has_image(obj):
        pass

    def format_date(date, format):
        pass

    def iso_date(date):
        pass

    def valid_date(date):
        pass

    def get_location(obj):
        pass

    def get_locations(obj):
        pass

    def get_youtube_url(obj):
        pass

    def clean_youtube_url(url):
        pass

    def get_uid(obj):
        pass

    def get_scale_url(content, scale):
        pass

    def get_main_links():
        pass

    def get_actions(name):
        pass

    def get_folder_section():
        pass

    def get_public_url():
        pass

    def render_breadcrumbs():
        pass

    def search():
        pass

    def is_anonymous():
        pass

    def site():
        pass

    def truncate_text():
        pass

    def get_popular_tags():
        pass

    def focal_image_tag():
        pass

    def normalize():
        pass

    def get_logo():
        pass

    def get_next_prev():
        pass

    def get_summary_text():
        pass

    def add_resource_on_request():
        pass

    def get_path():
        pass


class IDashboardUtils(Interface):
    def get_totals():
        pass

    def find_areas_of_interest():
        pass

    def sessions():
        pass

    def is_root():
        pass

    def get_path():
        pass

    def parse_ua():
        pass

    def get_in_review():
        pass

    def get_recently_modified():
        pass

    def get_recently_created():
        pass

    def get_creator():
        pass

    def get_modifier():
        pass

    def has_add_permission():
        pass

    def whois():
        pass



from castle.cms.interfaces import IReferenceNamedImage
from castle.cms.utils import has_image
from lxml.html import fromstring
from persistent.dict import PersistentDict
from persistent.mapping import PersistentMapping
from plone.app.drafts.utils import getCurrentDraft
from plone.app.uuid.utils import uuidToObject
from plone.namedfile.file import NamedBlobImage
from plone.tiles.data import ANNOTATIONS_KEY_PREFIX
from plone.uuid.interfaces import IUUID
from zope.annotation.interfaces import IAnnotations
from zope.interface import alsoProvides


def check_lead_image(obj, request=None):
    original_obj = obj
    if request is not None:
        # if request is passed in, we check for current draft to use...
        draft = getCurrentDraft(request)
        if draft is not None:
            obj = draft
    if has_image(obj):
        return
    image = find_image(obj)
    if image is not None:
        try:
            try:
                ct = image.image.contentType
            except AttributeError:
                ct = None
            value = NamedBlobImage(contentType=ct)
            alsoProvides(value, IReferenceNamedImage)
            value.reference = IUUID(image)
            original_obj.image = value
        except AttributeError:
            pass


def find_image_in_annotation(data):
    if type(data) in (dict, PersistentMapping, PersistentDict):
        for field_name in ('content', 'video', 'image', 'images', 'audio'):
            if field_name not in data:
                continue
            val = data[field_name]
            if not val:
                continue
            if isinstance(data, list):
                val = val[0]
            if not isinstance(val, basestring):
                continue
            val = val.strip()
            if '<' in val:
                # possible html...
                return find_image_in_html(val)
            else:
                im = uuidToObject(val)
                if im is not None and has_image(im):
                    return im


def find_image_in_html(html):
    try:
        dom = fromstring(html)
    except:
        # could not parse...
        return
    for img in dom.cssselect('img'):
        src = img.attrib.get('src')
        if not src:
            continue
        if 'resolveuid' in src:
            uid = src.split('resolveuid/')[-1].split('/@@images')[0]
            ob = uuidToObject(uid)
            if ob is not None:
                return ob


def find_image(obj):
    # get data from tile data
    annotations = IAnnotations(obj, {})
    for key in annotations.keys():
        if key.startswith(ANNOTATIONS_KEY_PREFIX):
            data = annotations[key]
            im = find_image_in_annotation(data)
            if im:
                return im
    if not getattr(obj, 'contentLayout', None) and getattr(obj, 'content'):
        im = find_image_in_html(obj.content)
        if im is not None:
            return im
    try:
        im = find_image_in_html(obj.text.raw)
        if im is not None:
            return im
    except AttributeError:
        pass



from castle.cms import impersonator
from castle.cms.theming import body_xpath
from chameleon import PageTemplate
from lxml.html import fromstring
from plone import api
from plone.transformchain.interfaces import ITransform
from Products.CMFPlone.log import logger
from repoze.xmliter.serializer import XMLSerializer
from zope.component import adapter
from zope.interface import implements
from ZPublisher.interfaces import IPubEnd
from ZPublisher.interfaces import IPubStart
from castle.cms.logger import log_request

import time


@adapter(IPubStart)
def requestStart(event):
    """
    """
    req = event.request
    req.environ['__started__'] = time.time()


@adapter(IPubEnd)
def requestEnd(event):
    req = event.request
    try:
        period = time.time() - req.environ['__started__']
    except:
        period = 0

    if period > 5.0:
        logger.warn('SLOW REQUEST(%i): %s' % (int(period), req.ACTUAL_URL))

    log_request(req)


_impersonator_template = PageTemplate("""
<div id="impersonator">
  <a href="${stop_url}" class="stop"><span class="glyphicon glyphicon-remove"></span></a>
  <span>This is what the website looks like to:
    <span class="anonymous" tal:condition="python: user_id == 'ANONYMOUS'">
        <span class="glyphicon glyphicon-globe"></span> Public
    </span>
    <span class="anonymous" tal:condition="python: user_id != 'ANONYMOUS'">
      <span class="glyphicon glyphicon-user"></span> ${user_name}
  </span>
</div>""")


class TransformInpersonatorOutput(object):
    implements(ITransform)

    order = 10000

    def __init__(self, published, request):
        self.published = published
        self.request = request

    def transformString(self, result, encoding):
        return None

    def transformUnicode(self, result, encoding):
        return None

    def impersonator(self, result):
        if not isinstance(result, XMLSerializer):
            return
        if impersonator.ORIGINAL_USER_KEY in self.request.environ:
            if api.user.is_anonymous():
                user_id = 'ANONYMOUS'
                user_name = 'Public'
            else:
                user = api.user.get_current()
                user_id = user.getId()
                user_name = user.getProperty('fullname') or user.getId()
            html = _impersonator_template(
                user_name=user_name, user_id=user_id,
                stop_url='%s/@@impersonator?action=stop&return_url=%s' % (
                    api.portal.get().absolute_url(),
                    self.request.URL))
            dom = fromstring(html)
            body = body_xpath(result.tree)
            if len(body) > 0:
                body[0].append(dom)
        return result

    def transformIterable(self, result, encoding):
        result = self.impersonator(result) or result
        return result



from castle.cms.theming import contentpanel_xpath
from castle.cms.utils import has_image
from lxml.html import tostring
from plone.app.blocks import gridsystem
from plone.app.blocks import tiles
from plone.app.blocks.layoutbehavior import ILayoutAware
from plone.app.blocks.utils import getLayout
from plone.namedfile.interfaces import INamedField
from plone.outputfilters import apply_filters
from plone.outputfilters.interfaces import IFilter
from plone.rfc822.interfaces import IPrimaryFieldInfo
from Products.CMFPlone.browser.syndication import adapters
from Products.CMFPlone.interfaces.syndication import IFeed
from repoze.xmliter.utils import getHTMLSerializer
from ZODB.POSException import POSKeyError
from zope.component import adapts
from zope.component import getAdapters
from zope.globalrequest import getRequest


class DexterityItem(adapters.DexterityItem):
    image = None
    file = None

    def __init__(self, context, feed):
        self.context = context
        self.feed = feed
        try:
            self._init()
        except (POSKeyError, AttributeError, TypeError):
            pass

    def _init(self):
        """
        moved here just so we can wrap it in exception block more nicely
        """
        try:
            primary = IPrimaryFieldInfo(self.context, None)
            if (INamedField.providedBy(primary.field) and
                    hasattr(primary.field, 'getSize') and
                    primary.field.getSize() > 0):
                self.file = primary.field
                self.field_name = primary.fieldname
        except TypeError:
            pass

        if has_image(self.context):
            self.image = self.file = self.context.image
            self.field_name = 'image'

    @property
    def file_length(self):
        if self.has_enclosure:
            try:
                return self.file.getSize()
            except POSKeyError:
                pass
        return 0

    @property
    def image_url(self):
        if self.image:
            return '{}/@@images/image'.format(self.base_url)


class LayoutAwareItem(DexterityItem):
    adapts(ILayoutAware, IFeed)

    def render_content_core(self, site=None):
        try:
            layout = getLayout(self.context)
        except TypeError:
            return super(LayoutAwareItem, self).render_content_core()
        req = getRequest()
        filters = [f for _, f
                   in getAdapters((self.context, req), IFilter)]
        layout = apply_filters(filters, layout)
        dom = getHTMLSerializer(layout)

        kwargs = {
            'baseURL': self.context.absolute_url() + '/layout_view'
        }
        if site is not None:
            kwargs['site'] = site

        tiles.renderTiles(req, dom.tree, **kwargs)
        gridsystem.merge(req, dom.tree)

        content = contentpanel_xpath(dom.tree)
        if len(content) > 0:
            return tostring(content[0])
        return ''



from castle.cms import subscribe
from castle.cms.constants import ALL_SUBSCRIBERS
from plone.registry.interfaces import IRegistry
from zope.component import getUtility

import json
import requests
import string


def send(message, numbers):
    if numbers == ALL_SUBSCRIBERS:
        numbers = subscribe.get_phone_numbers()

    if isinstance(numbers, basestring):
        numbers = [numbers]

    clean_numbers = []
    for number in numbers:
        number = ''.join(n for n in number if n in string.digits)
        if len(number) == 10:
            # assume at least american..
            number = '1' + number
        clean_numbers.append(number)

    registry = getUtility(IRegistry)
    auth_id = registry.get('castle.plivo_auth_id')
    auth_token = registry.get('castle.plivo_auth_token')
    src = registry.get('castle.plivo_phone_number')
    src = ''.join(n for n in src if n in string.digits)

    params = {
        'src': src,
        'dst': '<'.join(clean_numbers),
        'text': message
    }
    resp = requests.post(
        'https://api.plivo.com/v1/Account/%s/Message/' % auth_id,
        data=json.dumps(params),
        headers={
            'Content-Type': 'application/json'
        },
        auth=(auth_id, auth_token))
    if resp.status_code in (202, 200):
        try:
            return 'error' not in resp.json()
        except:
            return False
    else:
        return False



from plone.resource.manifest import MANIFEST_FILENAME
from plone.resource.manifest import getManifest
from plone.subrequest import subrequest
from plone.app.theming.browser.mapper import ThemeMapper as BaseThemeMapper
from castle.cms import theming
from plone import api
from plone.app.theming.utils import getTheme
from plone.app.theming.interfaces import MANIFEST_FORMAT


class ThemeMapper(BaseThemeMapper):

    def getFrame(self):
        theme_name = self.context.__name__
        theme = None
        if self.context.isFile(MANIFEST_FILENAME):
            manifest = self.context.openFile(MANIFEST_FILENAME)
            theme = getManifest(manifest, MANIFEST_FORMAT, None)
            theme['name'] = theme_name
            manifest.close()
        theme = getTheme(theme_name, theme)
        if theme.rules and theme.rules.endswith('.xml'):
            return super(ThemeMapper, self).getFrame()

        # we're assuming it is a castle type theme
        self.request.environ[theming.OVERRIDE_ENVIRON_KEY] = theme_name
        response = subrequest('/', root=api.portal.get())
        result = response.getBody()
        return result



from zope import schema
from zope.interface import Interface
from castle.cms.tiles.base import BaseTile
from castle.cms.vocabularies import AvailableFragments
from castle.cms.fragments.utils import getFragment


class FragmentTile(BaseTile):

    def render(self):
        fragment = self.data.get('fragment', '')
        if fragment:
            fragment = getFragment(
                self.context, self.request, fragment)
            options = {
                'tile': self
            }
            if hasattr(self, 'data_context'):
                options['data_context'] = self.data_context
            return fragment(options=options)
        return ''


class IFragmentTileSchema(Interface):

    fragment = schema.Choice(
        title=u"Fragment",
        source=AvailableFragments,
        required=True,
        default=None
    )



from castle.cms.interfaces import IAPISettings
from castle.cms.interfaces import IArchivalSettings
from castle.cms.interfaces import ICastleSettings
from castle.cms.interfaces import ISiteConfiguration
from castle.cms.widgets import SelectFieldWidget
from plone.app.registry.browser import controlpanel
from plone.formwidget.namedfile.widget import NamedFileFieldWidget
from Products.CMFPlone import PloneMessageFactory as _
from z3c.form import field
from z3c.form import group
from z3c.form.interfaces import HIDDEN_MODE


class APIForm(group.GroupForm):
    label = u"APIs"
    fields = field.Fields(IAPISettings)


class ArchivalForm(group.GroupForm):
    label = u"Archival"
    fields = field.Fields(IArchivalSettings)


class CastleSettingsControlPanelForm(controlpanel.RegistryEditForm):

    id = "CastleSettingsControlPanel"
    label = _(u"Castle Settings")
    schema = ICastleSettings
    schema_prefix = "castle"
    fields = field.Fields(ISiteConfiguration)
    groups = (APIForm, ArchivalForm)

    def updateFields(self):
        super(CastleSettingsControlPanelForm, self).updateFields()
        self.groups[0].fields['google_api_service_key_file'].widgetFactory = NamedFileFieldWidget
        self.groups[1].fields['archival_types_to_archive'].widgetFactory = SelectFieldWidget
        self.groups[1].fields['archival_states_to_archive'].widgetFactory = SelectFieldWidget

    def update(self):
        super(CastleSettingsControlPanelForm, self).update()
        self.groups[1].widgets["archival_replacements"].mode = HIDDEN_MODE


class CastleSettingsControlPanel(controlpanel.ControlPanelFormWrapper):
    form = CastleSettingsControlPanelForm



from Products.Five import BrowserView
from plone import api
import json


class FinishView(BrowserView):

    def __call__(self):
        # XXX this should be an ajax view

        if api.user.is_anonymous():
            return self.request.response.redirect(self.context.absolute_url())

        portal_memberdata = api.portal.get_tool('portal_memberdata')
        if not portal_memberdata.hasProperty("tours_viewed"):
            portal_memberdata.manage_addProperty(
                id="tours_viewed", value=[], type="lines")

        self.request.response.setHeader('Content-type', 'application/json')
        user = api.user.get_current()

        viewed = list(user.getProperty('tours_viewed', []))
        if not viewed:
            viewed = []
        viewed.append(self.request.form.get('tour'))
        viewed = list(set(viewed))
        user.setMemberProperties(mapping={'tours_viewed': viewed})
        return json.dumps({
            'success': True
        })


class TourAgainView(BrowserView):

    def __call__(self):
        if api.user.is_anonymous():
            return self.request.response.redirect(self.context.absolute_url())

        user = api.user.get_current()
        user.setMemberProperties(mapping={'tours_viewed': []})
        return self.request.response.redirect(self.context.absolute_url())



from zope.component import getUtility
from plone.registry.interfaces import IRegistry
from plone.app.users.browser import membersearch
from plone import api


class MemberSearchForm(membersearch.MemberSearchForm):

    def __call__(self):
        registry = getUtility(IRegistry)
        view_about = registry.get('plone.allow_anon_views_about', False)
        if not view_about and api.user.is_anonymous():
            return self.request.response.redirect(api.portal.get().absolute_url())
        return super(MemberSearchForm, self).__call__()
